#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


from casadi import *

# Read all about coloring in the seminal paper "What color is your Jacobian?" http://www.cs.odu.edu/~pothen/Papers/sirev2005.pdf

def color(A):
  print("="*80)
  print("Original:")
  print(repr(DM(A,1)))
  print("Colored: ")
  print(repr(DM(A.uni_coloring(),1)))

A = Sparsity.diag(5)
color(A)

# One direction needed to capture all

color(Sparsity.dense(5,10))

# We need 5 directions.
# The colored response reads: each row corresponds to a direction;
# each column correspond to a row of the original matrix.

color(A+Sparsity.triplet(5,5,[0],[4]))

# First 4 rows can be taken together, the fifth row is taken separately

color(A+Sparsity.triplet(5,5,[4],[0]))

# First 4 rows can be taken together, the fifth row is taken separately

color(A+Sparsity.triplet(5,5,[0]*5,list(range(5))))

# The first row is taken separately.
# The remainding rows are lumped together in one direction.

color(A+Sparsity.triplet(5,5,list(range(5)),[0]*5))

# We need 5 directions.

# Next, we look at star_coloring

def color(A):
  print("="*80)
  print("Original:")
  print(repr(DM(A,1)))
  print("Star colored: ")
  print(repr(DM(A.star_coloring(1),1)))
  
color(A)

# One direction needed to capture all

color(Sparsity.dense(5,5))

# We need 5 directions.

color(A+Sparsity.triplet(5,5,[0]*5,list(range(5)))+Sparsity.triplet(5,5,list(range(5)),[0]*5))

# The first row/col is taken separately.
# The remainding rows/cols are lumped together in one direction.

# Let's take an example from the paper

A = DM([[1,1,0,0,0,0],[1,1,1,0,1,1],[0,1,1,1,0,0],[0,0,1,1,0,1],[0,1,0,0,1,0],[0,1,0,1,0,1]])
A = sparsify(A)
color(A.sparsity())



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
import unittest
from types import *
from helpers import *

class Feasiblesqpmethod:#(casadiTestCase):

  def test_feas_iter_test(self):
      # Set options
      opts = {}
      opts["expand"] = True
      opts["hessian_approximation"] = "exact"
      opts["max_inner_iter"] = 30
      opts["max_iter"] = 20
      opts["feas_tol"] = 1e-6
      opts["tr_scale_vector"] = [1.0, 1.0] #[2.5,3.0,0.75}
      # opts["derivative_test"] = "second-order"

      # Specify QP solver
      opts["qpsol"]  = "nlpsol"
      opts["qpsol_options.nlpsol"] = "ipopt"
      opts["qpsol_options.error_on_fail"] = False
      opts["qpsol_options.nlpsol_options.ipopt.print_level"] = 0
      opts["qpsol_options.nlpsol_options.ipopt.sb"] = "yes"
      opts["qpsol_options.nlpsol_options.print_time"] = 0

      x = SX.sym("x")
      y = SX.sym("y")
      f = x**2 + (y-2)**2
      g = x**2 + y**2
      nlp = {"x": vertcat(x,y), "f": f, "g": g}

      # Create an NLP solver
      # Function solver = nlpsol("solver", "sqpmethod", nlp, opts)
      solver = nlpsol("solver", "feasiblesqpmethod", nlp, opts)

      # Solve the problem
      arg = {}
      arg["x0"] = [2.0, 0.0] #[2.5,3.0,0.75}
      arg["lbg"] = 4
      arg["ubg"] = 4
      res = solver(**arg)
      self.assertEqual(solver.stats()["iter_count"],9)
      self.checkarray(res["x"],vertcat(0,2),digits=7)

  def test_local_conv_test(self):
      """
      The problem contains a problem that contains a region where the
      local SQP method converges and a region from where it diverges. I.e., a 
      globalization strategy is necessary.
      """
      # Set options
      opts = {}
      opts["expand"] = True
      # opts["max_iter"] = 10)
      # opts["verbose"] = true
      # opts["linear_solver"] = "ma57"
      opts["hessian_approximation"] = "exact"
      # opts["derivative_test"] = "second-order"

      # Specify QP solver
      opts["qpsol"]  = "nlpsol"
      opts["qpsol_options.nlpsol"] = "ipopt"
      opts["qpsol_options.error_on_fail"] = False
      opts["qpsol_options.nlpsol_options.ipopt.print_level"] = 0
      opts["qpsol_options.nlpsol_options.ipopt.sb"] = "yes"
      opts["qpsol_options.nlpsol_options.print_time"] = 0

      x = SX.sym("x")
      f = log(exp(x) + exp(-x))
      g = x
      nlp = {"x": x, "f": f, "g": g}

      # Create an NLP solver
      # Function solver = nlpsol("solver", "sqpmethod", nlp, opts)
      solver = nlpsol("solver", "feasiblesqpmethod", nlp, opts)

      print("Here goes the non-converging initial point for local SQP")
      # Solve the problem
      arg = {}
      arg["x0"] = 1.4
      arg["lbg"] = -2
      arg["ubg"] = 2
      res = solver(**arg)
      self.assertEqual(solver.stats()["iter_count"],4)
      self.checkarray(res["x"],[0],digits=3)
      #uout() << "Optimal cost:                     " << double(res.at("f")) << std.endl
      #uout() << "Primal solution:                  " << std.vector<double>(res.at("x")) << std.endl
      #uout() << "Here goes the converging initial point for local SQP" << std.endl
      arg["x0"] = 1.0 #[2.5,3.0,0.75}
      res = solver(**arg)
      self.checkarray(res["x"],[0],digits=4)
      self.assertEqual(solver.stats()["iter_count"],2)

      #  Print solution
      #uout() << "Optimal cost:                     " << double(res.at("f")) << std.endl
      #uout() << "Primal solution:                  " << std.vector<double>(res.at("x")) << std.endl

  def test_qp_test(self):
      """
      The problem is a simple QP. Any SQP solver should just need one
      iteration! Depending on the size of the trust-region. It could take more 
      iteration. But if the trust-region is big enough such that the solution is
      included. The problem should just take one iteration!
      """
      for tr_rad0, iter_count in [(1.0,4),(100.0,2)]:
          # Set options
          opts = {}
          opts["expand"] = True
          # opts["max_iter"] = 10)
          # opts["verbose"] = true
          # opts["linear_solver"] = "ma57"
          opts["hessian_approximation"] = "exact"
          opts["tr_rad0"] = tr_rad0
          # opts["derivative_test"] = "second-order"

          # Specify QP solver
          opts["qpsol"]  = "nlpsol"
          opts["qpsol_options.nlpsol"] = "ipopt"
          opts["qpsol_options.error_on_fail"] = False
          opts["qpsol_options.nlpsol_options.ipopt.print_level"] = 0
          opts["qpsol_options.nlpsol_options.ipopt.sb"] = "yes"
          opts["qpsol_options.nlpsol_options.print_time"] = 0

          x = SX.sym("x")
          f = 2.0*(x+4)**2+2
          g = x
          nlp = {"x": x, "f": f, "g": g}

          # Create an NLP solver
          # Function solver = nlpsol("solver", "sqpmethod", nlp, opts)
          solver = nlpsol("solver", "feasiblesqpmethod", nlp, opts)

          # Solve the problem
          arg = {}
          arg["x0"] = 1.0 #[2.5,3.0,0.75}
          arg["lbg"] = -5
          arg["ubg"] = 5
          res = solver(**arg)
          self.assertEqual(solver.stats()["iter_count"],iter_count)
          self.checkarray(res["x"],[-4],digits=6)

if __name__ == '__main__':
    unittest.main()



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# -*- coding: utf-8 -*-
from casadi import *
from casadi.tools import *

import numpy as NP
import matplotlib.pyplot as plt

nk = 20    # Control discretization
tf = 10.0  # End time

# Declare variables (use scalar graph)
t  = SX.sym("t")    # time
u  = SX.sym("u")    # control

states = struct_symSX([
            entry('x',shape=2),    #  vdp oscillator states
            entry('L')             #  helper state: Langrange integrand
         ])

# Create a structure for the right hand side
rhs = struct_SX(states)
x = states['x']
rhs["x"] = vertcat((1 - x[1]*x[1])*x[0] - x[1] + u, x[0])
rhs["L"] = x[0]*x[0] + x[1]*x[1] + u*u

# ODE right hand side function
f = Function('f', [t,states,u],[rhs])

# Objective function (meyer term)
m = Function('m', [t,states,u],[states["L"]])

# Control bounds
u_min = -0.75
u_max = 1.0
u_init = 0.0

u_lb = NP.array([u_min])
u_ub = NP.array([u_max])
u_init = NP.array([u_init])

# State bounds and initial guess
x_min =  [-inf, -inf, -inf]
x_max =  [ inf,  inf,  inf]
xi_min = [ 0.0,  1.0,  0.0]
xi_max = [ 0.0,  1.0,  0.0]
xf_min = [ 0.0,  0.0, -inf]
xf_max = [ 0.0,  0.0,  inf]
x_init = [ 0.0,  0.0,  0.0]

# Dimensions
nx = 3
nu = 1

# Choose collocation points
tau_root = [0] + collocation_points(3,"radau")

# Degree of interpolating polynomial
d = len(tau_root)-1

# Size of the finite elements
h = tf/nk

# Coefficients of the collocation equation
C = NP.zeros((d+1,d+1))

# Coefficients of the continuity equation
D = NP.zeros(d+1)

# Dimensionless time inside one control interval
tau = SX.sym("tau")

# All collocation time points
T = NP.zeros((nk,d+1))
for k in range(nk):
  for j in range(d+1):
    T[k,j] = h*(k + tau_root[j])

# For all collocation points
for j in range(d+1):
  # Construct Lagrange polynomials to get the polynomial basis at the collocation point
  L = 1
  for r in range(d+1):
    if r != j:
      L *= (tau-tau_root[r])/(tau_root[j]-tau_root[r])

  # Evaluate the polynomial at the final time to get the coefficients of the continuity equation
  lfcn = Function('lfcn', [tau],[L])
  D[j] = lfcn(1.0)

  # Evaluate the time derivative of the polynomial at all collocation points to get the coefficients of the continuity equation
  tfcn = Function('tfcn', [tau],[tangent(L,tau)])
  for r in range(d+1):
    C[j,r] = tfcn(tau_root[r])

# Structure holding NLP variables
V = struct_symMX([
      (
       entry("X",repeat=[nk+1,d+1],struct=states),
       entry("U",repeat=[nk],shape=nu)
      )
    ])

vars_lb   = V()
vars_ub   = V()
vars_init = V()

# Set states and its bounds
vars_init["X",:,:] = repeated(repeated(x_init))
vars_lb["X",:,:]   = repeated(repeated(x_min))
vars_ub["X",:,:]   = repeated(repeated(x_max))

# Set controls and its bounds
vars_init["U",:] = repeated(u_init)
vars_lb["U",:]   = repeated(u_min)
vars_ub["U",:]   = repeated(u_max)

# State at initial time
vars_lb["X",0,0] = xi_min
vars_ub["X",0,0] = xi_max

# State at end time
vars_lb["X",-1,0] = xf_min
vars_ub["X",-1,0] = xf_max

# Constraint function for the NLP
g = []
lbg = []
ubg = []

# For all finite elements
for k in range(nk):

  # For all collocation points
  for j in range(1,d+1):

    # Get an expression for the state derivative at the collocation point
    xp_jk = 0
    for r in range (d+1):
      xp_jk += C[r,j]*V["X",k,r]

    # Add collocation equations to the NLP
    fk = f(T[k][j], V["X",k,j], V["U",k])
    g.append(h*fk - xp_jk)
    lbg.append(NP.zeros(nx)) # equality constraints
    ubg.append(NP.zeros(nx)) # equality constraints

  # Get an expression for the state at the end of the finite element
  xf_k = 0
  for r in range(d+1):
    xf_k += D[r]*V["X",k,r]

  # Add continuity equation to NLP
  g.append(V["X",k+1,0] - xf_k)
  lbg.append(NP.zeros(nx))
  ubg.append(NP.zeros(nx))

# Concatenate constraints
g = vertcat(*g)

# Objective function
f = m(T[nk-1][d],V["X",nk,0],V["U",nk-1])

# NLP
nlp = {'x':V, 'f':f, 'g':g}

## ----
## SOLVE THE NLP
## ----

# Set options
opts = {}
opts["expand"] = True
#opts["ipopt.max_iter"] = 4
opts["ipopt.linear_solver"] = 'ma27'

# Allocate an NLP solver
solver = nlpsol("solver", "ipopt", nlp, opts)
arg = {}

# Initial condition
arg["x0"] = vars_init

# Bounds on x
arg["lbx"] = vars_lb
arg["ubx"] = vars_ub

# Bounds on g
arg["lbg"] = NP.concatenate(lbg)
arg["ubg"] = NP.concatenate(ubg)

# Solve the problem
res = solver(**arg)

# Print the optimal cost
print("optimal cost: ", float(res["f"]))

# Retrieve the solution
opt = V(res["x"])

# Get values at the beginning of each finite element
x0_opt = vcat(opt["X",:,0,"x",0])
x1_opt = vcat(opt["X",:,0,"x",1])
x2_opt = vcat(opt["X",:,0,"L"])

u_opt = vcat(opt["U",:,0])

tgrid = NP.linspace(0,tf,nk+1)
tgrid_u = NP.linspace(0,tf,nk)

# Plot the results
plt.figure(1)
plt.clf()
plt.plot(tgrid,x0_opt,'--')
plt.plot(tgrid,x1_opt,'-.')
plt.step(tgrid_u,u_opt,'-')
plt.title("Van der Pol optimization")
plt.xlabel('time')
plt.legend(['x[0] trajectory','x[1] trajectory','u trajectory'])
plt.grid()
plt.show()



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#

from casadi import *
import numpy

# Let's construct a block diagonal structure

A = diagcat(1,DM([[2,3],[3,4]]),DM([[5,6,7],[6,8,9],[7,9,10]]),11)
print(A)
A.sparsity().spy()

numpy.random.seed(2)

# We randomly permute this nice structure

perm =  list(numpy.random.permutation(list(range(A.size1()))))
AP = A[perm,perm]

print(AP)
AP.sparsity().spy()

# And use scc to recover the blocks

n,p,r = AP.sparsity().scc()

APrestored = AP[p,p]

print(APrestored)
APrestored.sparsity().spy()
print("# blocks: ", n)
print("block boundaries: ", r[:n])



import glob
import subprocess
import os
import sys
import re

langs = ["python","octave"]

if len(sys.argv)>1:
  langs = sys.argv[1:]

success = True

for lang in langs:
    print("%s snippets" % lang)
    snippets = glob.glob('snippets/*.%s.in' % lang)
    for f in snippets:
       out_file = f.replace("in","out")
       if lang == "python":
         args = ["python",f]
       if lang == "octave":
         args = ["octave","--no-gui","--no-window-system","--eval","try,run('%s'),catch e,e.message,exit(1),end" % f]
     
       p = subprocess.run(args,stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True)
       with open(f.replace("in","out"), 'w') as myoutput:
       
         out = p.stdout
         # Filter out Ipopt copyright notice
         out = re.sub(r'\*{5,}.*Ipopt.*\*{5,}','',out,flags=re.MULTILINE|re.DOTALL)
         # Filter out qpOASES copyright notice
         out = re.sub(r"""qpOASES -- An Implementation of the Online Active Set Strategy.
Copyright \(C\) 2007-2015 by Hans Joachim Ferreau, Andreas Potschka,
Christian Kirches et al. All rights reserved.

qpOASES is distributed under the terms of the 
GNU Lesser General Public License 2.1 in the hope that it will be 
useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU Lesser General Public License for more details.""",'',out,flags=re.MULTILINE|re.DOTALL)

         # remove leading empty lines
         out_split = out.split("\n")
         for i,line in enumerate(out_split):
            line = line.rstrip()
            if len(line)!=0:
                out = "\n".join(out_split[i:])
                break
         
         myoutput.write(out)
       if p.returncode!=0:
         print(("="*5) + f)
         print(open(f,'r').read())
         print("="*50)
         print(open(out_file,'r').read())
         success = False
    print("Processed %d snippets" % len(snippets))

if not success:
  exit(1)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
import unittest
from types import *
from helpers import *
from time import time
import sys
from scipy import linalg, std, mean
from scipy.stats import t

student = t

class ComplexityTests(casadiTestCase):
  maxt = 0.4   # [s] Aim for execution times up to maxt
  mint = 0.04 # [s] Only trust execution times larger than mint
  rejectat = 0.01  # [-] Reject the null hypothesis if p-score is smaller than rejectat
  testorders = [0,1,2,3] # Test these orders
  debug = False
  check = True # Don't benchmark, just check for code errors
  def checkOrders(self,Ns,ts):
    """
    Test the hypothesis that the slope of Ns is order.


    An order -N means that the order is between (N-1) and N

    """
    # Test statistic http://stattrek.com/AP-Statistics-4/Test-Slope.aspx?Tutorial=Stat
    # http://mathworld.wolfram.com/LeastSquaresFitting.html
    # http://en.wikipedia.org/wiki/Student%27s_t-test#Slope_of_a_regression_line

    orders   = self.testorders
    rejectat = self.rejectat

    Ns = log(Ns)
    ts = log(ts)

    m = Ns.shape[0]

    sigmaN = std(Ns)
    sigmaT = std(ts)

    # covariance
    cov = mean(ts*Ns)-mean(ts)*mean(Ns)

    # correlation coefficient
    rho = (mean(ts*Ns)-mean(ts)*mean(Ns))/sigmaN/sigmaT

    # sqrt(Variance) estimate
    s = m/sqrt(m-2)*sqrt(sigmaT**2 - cov**2/sigmaN**2)

    # Estimate for b
    b = sigmaT/sigmaN * rho

    a = mean(ts) - b * mean(Ns)

    # Standard error on estimate for b
    sigmab = s/sqrt(m*sigmaN**2)

    # Standard deviation
    sigmaa = s * sqrt(1/m + mean(ts)**2/(m*sigmaT**2))

    conf = 0.05


    results = []
    for order in orders:
      # Null hypothesis: the slope of the regression is equal to order
      # test statistic
      t = abs(b-order)/sigmab

      p = (1-student.cdf(t,m))
      if self.debug:
        print("If the order were really %d, then the measurements have a p-score of %.8f" % (order,p))
      if p >= rejectat:
        results.append(order)


    if len(results)==1:
      order = results[0]
      a = mean(ts - order*Ns)
      sigmaa = std(ts - order*Ns)/sqrt(m)
      print("O(f) = %.3e N^%d [s]    | 95%% confidence:   [%.3e , %.3e] N^%d [s]" % (exp(a),order,exp(student.ppf(conf, m, loc=a, scale=sigmaa)),exp(student.ppf(1-conf, m, loc=a, scale=sigmaa)),order))
    else:
      print("raw fit O(f) = %.3e N^(%.3f) [s]" % (exp(a),b))


    for i, order in zip(list(range(len(orders)))[1:],orders[1:]):
      if b < order and b > order-1 and not(order in results) and not(order -1 in results):
        results.append(-order)

    return results


  def complexity(self,setupfun,fun, order, depth = 0):

    N = 1

    Ns = []
    ts = []

    dt = 0
    while dt<self.maxt:
      sys.stdout.write("\r%d..." % N)
      sys.stdout.flush()
      p = setupfun(self,N) # Setup
      for i in range(10):
        t = time()
        fun(self,N,p) # Run the function
        dt = time()-t
        Ns.append(N)
        ts.append(dt)
      N=int(N*1.5)
      N+=1
      if self.check: break
    print("")

    Ns = array(Ns)
    ts = array(ts)
    valid = ts > self.mint
    if not(self.check):
      orders = self.checkOrders(Ns[valid],ts[valid])
    if not(self.check) and (len(orders)!=1 or orders[0]!=order):
      if (depth<3):
        return self.complexity(setupfun,fun, order, depth+1 )
      else:
        self.assertTrue(False,"We expected order %d, but found %s" % (order,str(orders)))


  def test_DMadd(self):
    self.message("DM add column vectors")
    def setupfun(self,N):
      return {'A': DM(N,1,0), 'B': DM(N,1,0)}
    def fun(self,N,setup):
      setup['A'] + setup['B']

    self.complexity(setupfun,fun, 1)

    self.message("DM add rows vectors")
    def setupfun(self,N):
      return {'A': DM(1,N,0), 'B': DM(1,N,0)}

    self.complexity(setupfun,fun, 1)


  def test_SX_funadd(self):
    return
    self.message("SX add column vectors")
    def setupfun(self,N):
      A = SX.sym("A",N,1)
      B = SX.sym("B",N,1)
      f = Function('f', [A,B],[A+B])
      return {'f':f}
    def fun(self,N,setup):
      setup['f'].evaluate()

    self.complexity(setupfun,fun, 1)

  def test_SX_funprodvec(self):
    return
    self.message("SX prod column vectors")
    def setupfun(self,N):
      A = SX.sym("A",N,1)
      B = SX.sym("B",N,1)
      f = Function('f', [A,B],[c.dot(A.T,B)])
      return {'f':f}
    def fun(self,N,setup):
      setup['f'].evaluate()
    self.complexity(setupfun,fun, 1)

  def test_SX_funprodsparse(self):
    self.message("SX prod sparse")
    def setupfun(self,N):
      A = SX.sym("A",Sparsity.diag(N))
      A[-1,0]=SX("off") # Have one of-diagonal element
      B = SX.sym("B",N,1)
      f = Function('f', [A,B],[c.dot(A,B)])
      return {'f':f}
    def fun(self,N,setup):
      setup['f'].evaluate()

    self.complexity(setupfun,fun, 1)


  def test_MX_funprodvec(self):
    self.message("MX prod")
    def setupfun(self,N):
      G = MX.sym("G",N,1)
      X = MX.sym("X",N,1)
      f = Function('f', [G,X],[c.prod(G.T,X)])
      return {'f':f}
    def fun(self,N,setup):
      setup['f'].evaluate()
    self.complexity(setupfun,fun, 1)

  def test_MX_funprodsparse(self):
    self.message("MX sparse product")
    def setupfun(self,N):
      s = Sparsity.diag(N)
      s[-1,0]=1
      H = MX.sym("H",s)
      X = MX.sym("X",N,1)
      f = Function('f', [H,X],[c.prod(H,X)])
      return {'f':f}
    def fun(self,N,setup):
      setup['f'].evaluate()
    self.complexity(setupfun,fun, 2)  # 1
    self.message("MX sparse sparse product")
    def setupfun(self,N):
      s = Sparsity.diag(N)
      s[-1,0]=1
      H = MX.sym("H",s)
      s = Sparsity.diag(N)
      s[-1,0]=1
      X = MX.sym("X",s)
      f = Function('f', [H,X],[c.prod(H,X)])
      return {'f':f}
    self.complexity(setupfun,fun, 2)  # 1

  def test_DMdot(self):
    self.message("DM inner dot vectors")
    def setupfun(self,N):
      return {'A': DM(1,N,0), 'B': DM(N,1,0)}
    def fun(self,N,setup):
      c.dot(setup['A'],setup['B'])

    self.complexity(setupfun,fun, 1)

    self.message("DM outer dot vectors")
    def setupfun(self,N):
      return {'A': DM(N,1,0), 'B': DM(1,N,0)}
    def fun(self,N,setup):
      c.dot(setup['A'],setup['B'])

    self.complexity(setupfun,fun, 2) # strangely, the dot product is O(N^2)




if __name__ == '__main__':
    unittest.main()




# This is an automatically generated file converting from the apm format

import casadi as cs

def hs077():
    # The optimal objective is (if given in):
    f_opt = 0.24150513
    x_opt = cs.DM([1.16617, 1.18211, 1.38026, 1.50604, 0.61092])
    x = cs.MX.sym('x', 5)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(5, 1)
    lbx = -cs.inf*cs.DM.ones(5, 1)
    ubx = cs.inf*cs.DM.ones(5, 1)
    g = cs.MX.zeros(2, 1)
    lbg = -cs.inf*cs.DM.ones(2, 1)
    ubg = cs.inf*cs.DM.ones(2, 1)
    
    x0[0:5]= 2
    
    lbg[0] = 2*cs.sqrt(2)
    ubg[0] = 2*cs.sqrt(2)
    g[0] = x[0]**2*x[3] + cs.sin(x[3]-x[4])
    lbg[1] = 8 + cs.sqrt(2)
    ubg[1] = 8 + cs.sqrt(2)
    g[1] = x[1] + x[2]**4*x[3]**2  
    obj = (x[0]-1)**2 + (x[0] - x[1])**2 + (x[2]-1)**2 + (x[3]-1)**4 + (x[4]-1)**6
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
from casadi import *
from pylab import *

# Control
u = MX.sym("u")

# State
x = MX.sym("x",3)
s = x[0] # position
v = x[1] # speed
m = x[2] # mass

# ODE right hand side
sdot = v
vdot = (u - 0.05 * v*v)/m
mdot = -0.1*u*u
xdot = vertcat(sdot,vdot,mdot)

# ODE right hand side function
f = Function('f', [x,u],[xdot])

# Integrate with Explicit Euler over 0.2 seconds
dt = 0.01  # Time step
xj = x
for j in range(20):
  fj = f(xj,u)
  xj += dt*fj

# Discrete time dynamics function
F = Function('F', [x,u],[xj])

# Number of control segments
nu = 50 

# Control for all segments
U = MX.sym("U",nu) 
 
# Initial conditions
X0 = MX([0,0,1])

# Integrate over all intervals
X=X0
for k in range(nu):
  X = F(X,U[k])

# Objective function and constraints
J = mtimes(U.T,U) # u'*u in Matlab
G = X[0:2]     # x(1:2) in Matlab

# NLP
nlp = {'x':U, 'f':J, 'g':G}
 
# Allocate an NLP solver
opts = {"ipopt.tol":1e-10, "expand":True}
solver = nlpsol("solver", "ipopt", nlp, opts)
arg = {}

# Bounds on u and initial condition
arg["lbx"] = -0.5
arg["ubx"] =  0.5
arg["x0"] =   0.4

# Bounds on g
arg["lbg"] = [10,0]
arg["ubg"] = [10,0]

# Solve the problem
res = solver(**arg)

# Get the solution
plot(res["x"])
plot(res["lam_x"])
grid()
show()



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# -*- coding: utf-8 -*-
from casadi import *

# Example how to export FMUs from CasADi
# Joel Andersson, joel@jaeandersson.com

# Start with an empty DaeBuilder instance
dae = DaeBuilder('vdp')

# States
x1 = dae.add_x('x1')
x2 = dae.add_x('x2')
q = dae.add_q('q')

# Input
u = dae.add_u('u')

# Set ODE right-hand-sides
dae.set_ode('x1', (1 - x2 * x2)*x1 - x2 + u)
dae.set_ode('x2', x1)
dae.set_ode('q', x1**2 + x2**2 + u**2)

# Specify initial conditions
dae.set_start('x1', 0)
dae.set_start('x2', 0)

# Add bounds
dae.set_min('u', -0.75)
dae.set_max('u', 1)
dae.set_start('u', 0)

# Print DAE
dae.disp(True)

# Export FMU
funcs = dae.export_fmu()
print('Generated files: {}'.format(funcs))

# Compile DLL
import os
os.system('gcc --shared -fPIC -I../../../external_packages/FMI-Standard-3.0/headers/ vdp.c vdp_wrap.c -o vdp.so')
print('Compiled vdp.so')

# Package into an FMU
import zipfile
with zipfile.ZipFile('vdp_generated.fmu', 'w') as fmufile:
    # Add generated files to the archive
    for f in funcs:
      arcname = f if f == 'modelDescription.xml' else 'sources/' + f
      fmufile.write(f, arcname = arcname)
      os.remove(f)
    # Add compile DLL to the archive (assume Linux 64 but)
    fmufile.write('vdp.so', arcname = 'binaries/x86-linux/vdp.so')
    os.remove('vdp.so')
print('Created FMU: vdp_generated.fmu')



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# -*- coding: utf-8 -*-
from casadi import *
import numpy as NP
import matplotlib.pyplot as plt

# Degree of interpolating polynomial
d = 3

# Choose collocation points
tau_root = [0] + collocation_points(d, "radau")

# Coefficients of the collocation equation
C = NP.zeros((d+1,d+1))

# Coefficients of the continuity equation
D = NP.zeros(d+1)

# Coefficients of the quadrature function
F = NP.zeros(d+1)

# Construct polynomial basis
for j in range(d+1):
  # Construct Lagrange polynomials to get the polynomial basis at the collocation point
  p = NP.poly1d([1])
  for r in range(d+1):
    if r != j:
      p *= NP.poly1d([1, -tau_root[r]]) / (tau_root[j]-tau_root[r])
  
  # Evaluate the polynomial at the final time to get the coefficients of the continuity equation
  D[j] = p(1.0)

  # Evaluate the time derivative of the polynomial at all collocation points to get the coefficients of the continuity equation
  pder = NP.polyder(p)
  for r in range(d+1):
    C[j,r] = pder(tau_root[r])

  # Evaluate the integral of the polynomial to get the coefficients of the quadrature function
  pint = NP.polyint(p)
  F[j] = pint(1.0)

# Control discretization
nk = 20

# End time
tf = 10.0  

# Size of the finite elements
h = tf/nk

# All collocation time points
T = NP.zeros((nk,d+1))
for k in range(nk):
  for j in range(d+1):
    T[k,j] = h*(k + tau_root[j])

# Declare variables (use scalar graph)
t  = SX.sym("t")    # time
u  = SX.sym("u")    # control
x  = SX.sym("x",2)  # state

# ODE rhs function and quadratures
xdot = vertcat((1 - x[1]*x[1])*x[0] - x[1] + u, \
               x[0])
qdot = x[0]*x[0] + x[1]*x[1] + u*u
f = Function('f', [t,x,u],[xdot, qdot])

# Control bounds
u_min = -0.75
u_max = 1.0
u_init = 0.0

u_lb = NP.array([u_min])
u_ub = NP.array([u_max])
u_init = NP.array([u_init])

# State bounds and initial guess
x_min =  [-inf, -inf]
x_max =  [ inf,  inf]
xi_min = [ 0.0,  1.0]
xi_max = [ 0.0,  1.0]
xf_min = [ 0.0,  0.0]
xf_max = [ 0.0,  0.0]
x_init = [ 0.0,  0.0]

# Dimensions
nx = 2
nu = 1

# Total number of variables
NX = nk*(d+1)*nx      # Collocated states
NU = nk*nu              # Parametrized controls
NXF = nx                # Final state
NV = NX+NU+NXF

# NLP variable vector
V = MX.sym("V",NV)
  
# All variables with bounds and initial guess
vars_lb = NP.zeros(NV)
vars_ub = NP.zeros(NV)
vars_init = NP.zeros(NV)
offset = 0

# Get collocated states and parametrized control
X = NP.resize(NP.array([],dtype=MX),(nk+1,d+1))
U = NP.resize(NP.array([],dtype=MX),nk)
for k in range(nk):  
  # Collocated states
  for j in range(d+1):
    # Get the expression for the state vector
    X[k,j] = V[offset:offset+nx]
    
    # Add the initial condition
    vars_init[offset:offset+nx] = x_init
    
    # Add bounds
    if k==0 and j==0:
      vars_lb[offset:offset+nx] = xi_min
      vars_ub[offset:offset+nx] = xi_max
    else:
      vars_lb[offset:offset+nx] = x_min
      vars_ub[offset:offset+nx] = x_max
    offset += nx
  
  # Parametrized controls
  U[k] = V[offset:offset+nu]
  vars_lb[offset:offset+nu] = u_min
  vars_ub[offset:offset+nu] = u_max
  vars_init[offset:offset+nu] = u_init
  offset += nu
  
# State at end time
X[nk,0] = V[offset:offset+nx]
vars_lb[offset:offset+nx] = xf_min
vars_ub[offset:offset+nx] = xf_max
vars_init[offset:offset+nx] = x_init
offset += nx
  
# Constraint function for the NLP
g = []
lbg = []
ubg = []

# Objective function
J = 0

# For all finite elements
for k in range(nk):
  
  # For all collocation points
  for j in range(1,d+1):
        
    # Get an expression for the state derivative at the collocation point
    xp_jk = 0
    for r in range (d+1):
      xp_jk += C[r,j]*X[k,r]
      
    # Add collocation equations to the NLP
    fk,qk = f(T[k,j], X[k,j], U[k])
    g.append(h*fk - xp_jk)
    lbg.append(NP.zeros(nx)) # equality constraints
    ubg.append(NP.zeros(nx)) # equality constraints

    # Add contribution to objective
    J += F[j]*qk*h

  # Get an expression for the state at the end of the finite element
  xf_k = 0
  for r in range(d+1):
    xf_k += D[r]*X[k,r]

  # Add continuity equation to NLP
  g.append(X[k+1,0] - xf_k)
  lbg.append(NP.zeros(nx))
  ubg.append(NP.zeros(nx))
  
# Concatenate constraints
g = vertcat(*g)
  
# NLP
nlp = {'x':V, 'f':J, 'g':g}

## ----
## SOLVE THE NLP
## ----

# Set options
opts = {}
opts["expand"] = True
#opts["ipopt.max_iter"] = 4
opts["ipopt.linear_solver"] = 'ma27'

# Allocate an NLP solver
solver = nlpsol("solver", "ipopt", nlp, opts)
arg = {}
  
# Initial condition
arg["x0"] = vars_init

# Bounds on x
arg["lbx"] = vars_lb
arg["ubx"] = vars_ub

# Bounds on g
arg["lbg"] = NP.concatenate(lbg)
arg["ubg"] = NP.concatenate(ubg)

# Solve the problem
res = solver(**arg)

# Print the optimal cost
print("optimal cost: ", float(res["f"]))

# Retrieve the solution
v_opt = NP.array(res["x"])

# Get values at the beginning of each finite element
x0_opt = v_opt[0::(d+1)*nx+nu]
x1_opt = v_opt[1::(d+1)*nx+nu]
u_opt = v_opt[(d+1)*nx::(d+1)*nx+nu]
tgrid = NP.linspace(0,tf,nk+1)
tgrid_u = NP.linspace(0,tf,nk)

# Plot the results
plt.figure(1)
plt.clf()
plt.plot(tgrid,x0_opt,'--')
plt.plot(tgrid,x1_opt,'-.')
plt.step(tgrid_u,u_opt,'-')
plt.title("Van der Pol optimization")
plt.xlabel('time')
plt.legend(['x[0] trajectory','x[1] trajectory','u trajectory'])
plt.grid()
plt.show()




#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
import unittest
from types import *
from helpers import *
import pickle
from operator import itemgetter
import sys
from casadi.tools import capture_stdout

scipy_available = True
try:
    import scipy.special
    from scipy.linalg import expm
except:
    scipy_available = False

class Misctests(casadiTestCase):

  def test_issue179B(self):
    self.message('Regression test #179 (B)')
    def calc_sparsity():
      x = casadi.SX.sym("x")
      f = casadi.Function('f', [x], [x ** 2])
      return f.jac_sparsity(0, 0)

    def print_sparsity():
        sparsity = calc_sparsity()
        str(sparsity) # Segfault

    print_sparsity()

  def test_sanity(self):
    DM(Sparsity(4,3,[0,2,2,3],[1,2,1]),[0.738,0.39,0.99])
    self.assertRaises(RuntimeError,lambda : DM(Sparsity(4,4,[0,2,2,3],[1,2,1]),[0.738,0.39,0.99]))
    self.assertRaises(RuntimeError,lambda : DM(Sparsity(4,3,[0,2,2,12],[1,2,1]),[0.738,0.39,0.99]))
    self.assertRaises(RuntimeError,lambda : DM(Sparsity(4,3,[-10,2,2,3],[1,2,1]),[0.738,0.39,0.99]))
    self.assertRaises(RuntimeError,lambda : DM(Sparsity(4,3,[0,2,2,3],[8,2,1]),[0.738,0.39,0.99]))
    self.assertRaises(RuntimeError,lambda : DM(Sparsity(4,3,[0,2,2,3],[-3,2,1]),[0.738,0.39,0.99]))
    self.assertRaises(RuntimeError,lambda : DM(Sparsity(4,3,[0,2,2,3],[1,2,1,2]),[0.738,0.39,0.99]))
    self.assertRaises(RuntimeError,lambda : DM(Sparsity(4,3,[0,2,0,3],[1,2,1]),[0.738,0.39,0.99]))

  def test_copyconstr_norefcount(self):
    self.message("Copy constructor for non-refcounted classes")
    x = DM.ones(2,3)

    y = DM(x)
    x[0,0] = 5

    self.assertFalse(id(x)==id(y))
    self.assertEqual(x[0,0],5)
    self.assertEqual(y[0,0],1)

  def test_copyconstr_refcount(self):
    self.message("Copy constructor for refcounted classes")
    x = Sparsity.diag(4)

    y = Sparsity(x)

    x.resize(2,8)

    self.assertFalse(id(x)==id(y))
    self.assertTrue(x.numel(),y.numel())
    self.checkarray(x.shape,(2,8),"shape")
    self.checkarray(y.shape,(4,4),"shape")

  def test_copy_norefcount(self):
    self.message("Shallow copy for non-refcounted classes")
    import copy

    x = DM.ones(2,3)

    y = copy.copy(x)
    x[0,0] = 5

    self.assertFalse(id(x)==id(y))
    self.assertEqual(x[0,0],5)
    self.assertEqual(y[0,0],1)

  def test_copy_refcount(self):
    self.message("Shallow copy for refcounted classes")
    import copy
    x = Sparsity.diag(4)

    y = copy.copy(x)

    x.resize(2,8)

    self.assertFalse(id(x)==id(y))
    self.assertTrue(x.numel(),y.numel())
    self.checkarray(x.shape,(2,8),"shape")
    self.checkarray(y.shape,(4,4),"shape")

  def test_deepcopy_norefcount(self):
    self.message("Deep copy for non-refcounted classes")
    import copy

    x = DM.ones(2,3)

    y = copy.deepcopy(x)
    x[0,0] = 5

    self.assertFalse(id(x)==id(y))
    self.assertEqual(x[0,0],5)
    self.assertEqual(y[0,0],1)

  def test_deepcopy_refcount(self):
    self.message("Deep copy for refcounted classes")
    import copy
    x = Sparsity.diag(4)

    y = copy.deepcopy(x)

    x.resize(2,8)

    self.assertFalse(id(x)==id(y))
    self.assertTrue(x.numel(),y.numel())
    self.checkarray(x.shape,(2,8),"shape")
    self.checkarray(y.shape,(4,4),"shape")

  @requiresPlugin(nlpsol,"ipopt")
  def test_options_introspection(self):
    self.message("options introspection")
    x=SX.sym("x")
    nlp = {'x':x, 'f':x**2}
    i = nlpsol('i', "ipopt", nlp)

    opts = i.optionNames()
    self.assertTrue(isinstance(opts,list))

    n = opts[0]
    self.assertTrue(type(n)==type(""))

    n = "monitor"

    d = i.optionDescription(n)
    self.assertTrue(type(d)==type(""))
    self.assertTrue(not("d"=="N/A"))

    d = i.optionTypeName(n)
    self.assertEqual(d,"OT_STRINGVECTOR")

    #d = i.optionAllowed(n)

  def test_pickling(self):

    a = Sparsity.lower(4)
    s = pickle.dumps(a)
    b = pickle.loads(s)
    self.assertTrue(a==b)

    a = Sparsity()
    s = pickle.dumps(a)
    b = pickle.loads(s)
    self.assertTrue(a.is_null())

    a = DM(Sparsity.lower(4),list(range(10)))
    s = pickle.dumps(a)
    b = pickle.loads(s)
    self.checkarray(a,b)

  def test_exceptions(self):
    if systemswig: return

    try:
      nlpsol(123)
      self.assertTrue(False)
    except NotImplementedError as e:
      e_message = str(e)
      assert "nlpsol(str,str,dict:MX,dict)" in e_message
      assert "You have: '(int)'" in e_message
      assert "::" not in e_message
      assert "std" not in e_message

    try:
      vcat(123)
      self.assertTrue(False)
    except NotImplementedError as e:
      e_message = str(e)
      assert "vertcat([SX]" in e_message
      assert "vertcat([DM]" in e_message
      assert "You have: '(int)'" in e_message
      assert "::" not in e_message
      assert "std" not in e_message

    try:
      substitute(123)
      self.assertTrue(False)
    except NotImplementedError as e:
      e_message = str(e)
      assert "substitute(SX,SX,SX)" in e_message
      assert "substitute([SX],[SX],[SX])" in e_message
      assert "You have: '(int)'" in e_message
      assert "::" not in e_message
      assert "std" not in e_message

    try:
      load_nlpsol(132)
      self.assertTrue(False)
    except NotImplementedError as e:
      e_message = str(e)
      assert "load_nlpsol(str)" in e_message
      assert "::" not in e_message
      assert "std" not in e_message

    x=SX.sym("x")

    try:
      [x]+ x
      self.assertTrue(False)
    except TypeError as e:
      e_message = str(e)

    try:
      x + [x]
      self.assertTrue(False)
    except TypeError as e:
      e_message = str(e)

    try:
      x.reshape(2)
      self.assertTrue(False)
    except NotImplementedError as e:
      e_message = str(e)
      assert "reshape(SX,(int,int))" in e_message

    try:
      x.reshape(("a",2))
      self.assertTrue(False)
    except NotImplementedError as e:
      e_message = str(e)
      assert "You have: '(SX,(str,int))'" in e_message

    try:
      diagsplit("s")
      self.assertTrue(False)
    except NotImplementedError as e:
      e_message = str(e)
      assert "diagsplit(SX,int)" in e_message
      assert "diagsplit(DM,int)" in e_message

    try:
      DM("df")
      self.assertTrue(False)
    except NotImplementedError as e:
      e_message = str(e)
      assert "  DM(" in e_message

    try:
      vertcat(1,SX.sym('x'),MX.sym('x'))
      self.assertTrue(False)
    except NotImplementedError as e:
      e_message = str(e)
      assert "vertcat(" in e_message

  def test_getscheme(self):
    x = SX.sym("x")
    p = SX.sym("p")

    F = Function('F', [x, p], [x+p, x**2], ['x', 'p'], ['f', 'g'])

    fc = F(x=3,p=4)
    f = fc['f']
    self.checkarray(f,DM([7]))
    g = fc['g']
    self.checkarray(g,DM([9]))
    [f,g] = itemgetter('f','g')(fc)
    self.checkarray(f,DM([7]))
    self.checkarray(g,DM([9]))
    [g,f] = itemgetter('g','f')(fc)
    self.checkarray(f,DM([7]))
    self.checkarray(g,DM([9]))

  def test_assertions(self):

    x = MX.sym("x")

    z = x**2

    z = z.attachAssert(z>3,"x must be larger than 3")

    v = sin(z)

    f = Function('f', [x],[v])

    print(f)

    f_out = f(-6)
    try :
      f_out = f(1)
    except Exception as e:
      print(str(e))
      self.assertTrue("x must be larger than 3" in str(e))

  def test_doc(self):
    self.assertTrue("jac_penalty" in nlpsol.__doc__) # FunctionInternal
    self.assertTrue("print_time" in nlpsol.__doc__)  # ProtoFunction
    self.assertTrue( nlpsol.__doc__.count("print_time")==1)  # ProtoFunction

  @requires_nlpsol("ipopt")
  def test_output(self):
    with capture_stdout() as result:
      DM([1,2]).print_dense()


    assert "2" in result[0]

    x=SX.sym("x")
    f = {'x':x, 'f':x**2}
    solver = nlpsol("solver", "ipopt",f)
    with capture_stdout() as result:
      solver_out = solver(x0=0)

    assert "Number of nonzeros in equality constraint" in result[0]
    assert "iter    objective    inf_pr" in result[0]

    with capture_stdout() as result:
      try:
        solver = nlpsol("solver","foo",f)
      except:
        pass

    assert "casadi_nlpsol_foo" in result[1]

  def test_serialize(self):

    x = Sparsity.upper(3)

    y = SX.sym("y") # nested

    si = StringSerializer()
    si.pack([x])

    z = y
    for i in range(10000):
      z = sin(z)
    e = vertcat(y,z,2*z)
    fref = Function('f',[y],[e])
    si.pack(e)
    data = si.encode()
    si.pack([x])

    si = StringDeserializer(data)

    spx = si.unpack()
    self.assertTrue(spx[0]==x)
    A = si.unpack()
    f = Function('f',[A[0]],[A])
    self.checkfunction_light(f,fref,[7])
    with self.assertInException("end of stream"):
      si.unpack()
    with self.assertInException("end of stream"):
      si.unpack()

    si = FileSerializer("foo.dat")
    si.pack([x])
    z = sin(y)
    e = vertcat(y,z,2*z)
    fref = Function('f',[y],[e])
    si.pack(e)
    si = None
    si = FileDeserializer("foo.dat")

    spx = si.unpack()
    self.assertTrue(spx[0]==x)
    A = si.unpack()
    f = Function('f',[A[0]],[A])
    self.checkfunction_light(f,fref,[7])
    with self.assertInException("end of stream"):
      si.unpack()
    with self.assertInException("end of stream"):
      si.unpack()


    x = SX.sym('x')
    s = StringSerializer()
    s.pack(x)
    data1 = s.encode()
    s.pack(sin(x))
    data2 = s.encode()

    s = StringDeserializer(data1)
    a = s.unpack()
    with self.assertInException("end of stream"):
      s.unpack()
    s.decode(data2)
    b = s.unpack()
    with self.assertInException("end of stream"):
      s.unpack()

    si = FileSerializer("foo.dat")
    f = Function("f",[x],[x**2])
    g = Function("g",[x],[x**3])
    si.pack([f,g])
    si = None
    with self.assertInException("File is not loadable with 'load'. Use 'FileDeserializer' instead."):
      r = Function.load("foo.dat")

    si = FileDeserializer("foo.dat")
    print(si.unpack())

    si = FileSerializer("foo.dat")
    f = Function("f",[x],[x**2])
    si.pack(f)
    si = None
    r = Function.load("foo.dat")
    print(r)

    f.save("foo.dat")
    si = FileDeserializer("foo.dat")
    print(si.unpack())

  def test_print_time(self):


    x = MX.sym("x")
    f = x**2


    for print_time in [True,False]:
      included = ["t_wall"] if print_time else []
      excluded = [] if print_time else ["t_wall"]
      opts = {"print_time":print_time}
      ff = Function("f",[x],[f],opts)
      with self.assertOutput(included, excluded):
        ff(3)

      if has_nlpsol("ipopt"):
        solver = nlpsol("nlpsol","ipopt",{"x":x,"f":f},opts)
        with self.assertOutput(included, excluded):
          solver()


      if has_conic("ipopt"):
        solver = qpsol("qpsol","qpoases",{"x":x,"f":f},opts)
        with self.assertOutput(included, excluded):
          solver()

      solver = rootfinder("rootfinder","newton",{"x":x,"g":x},opts)
      with self.assertOutput(included, excluded):
        solver()

      solver = integrator("integrator","rk",{"x":x,"ode":f},opts)
      with self.assertOutput(included, excluded):
        solver()


      integr_options = dict(opts)
      integr_options["simplify"] = True
      solver = integrator("integrator","rk",{"x":x,"ode":f},integr_options)
      with self.assertOutput(included, excluded):
        solver()

      A = DM.rand(3,3)
      if has_linsol("lapacklu"):
        with self.assertOutput(included, excluded):
          solve(A,vertcat(1,2,3),"lapacklu",opts)


      Amx = MX(DM.rand(3,3))
      if has_linsol("lapacklu"):
        with self.assertOutput(included, excluded):
          evalf(solve(Amx,vertcat(1,2,3),"lapacklu",opts))

      if has_linsol("lapacklu"):
        solver = Linsol("linsol","lapacklu",A.sparsity(),opts)
        with self.assertOutput(included, excluded):
          solver.solve(A,vertcat(1,2,3))

  @memory_heavy()
  def test_record_time(self):


    x = MX.sym("x")
    f = x**2
    for i in range(10000):
      f =sin(f)*f
    f=f+x**2

    opts = {"record_time":True}
    ff = Function("f",[x],[f],opts)
    ff(3)
    self.assertTrue("t_proc_total" in ff.stats())
    self.assertTrue(ff.stats()["t_wall_total"]>=0)

    if has_nlpsol("ipopt"):
      solver = nlpsol("nlpsol","ipopt",{"x":x,"f":f},opts)
      solver()
      self.assertTrue("t_proc_total" in solver.stats())
      self.assertTrue(solver.stats()["t_wall_total"]>0)

    if has_conic("qpoases"):
      solver = qpsol("qpsol","qpoases",{"x":x,"f":f},opts)
      solver()
      self.assertTrue("t_proc_total" in solver.stats())
      self.assertTrue(solver.stats()["t_wall_total"]>0)

    solver = rootfinder("rootfinder","newton",{"x":x,"g":x},opts)
    solver()
    self.assertTrue("t_proc_total" in solver.stats())
    self.assertTrue(solver.stats()["t_wall_total"]>=0)

    solver = integrator("integrator","rk",{"x":x,"ode":f},opts)
    solver()
    self.assertTrue("t_proc_total" in solver.stats())
    self.assertTrue(solver.stats()["t_wall_total"]>=0)

    integr_options = {}
    integr_options["simplify"] = True
    integr_options["simplify_options"] = opts
    solver = integrator("integrator","rk",{"x":x,"ode":f},integr_options)
    solver()
    self.assertTrue("t_proc_total" in solver.stats())
    self.assertTrue(solver.stats()["t_wall_total"]>=0)

    A = DM.rand(3,3)
    if has_linsol("lapacklu"):
      solver = Linsol("linsol","lapacklu",A.sparsity(),opts)
      solver.solve(A,vertcat(1,2,3))
      self.assertTrue("t_proc_total" in solver.stats())
      self.assertTrue(solver.stats()["t_wall_total"]>=0)

if __name__ == '__main__':
    unittest.main()



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
from casadi import *

"""
Solve the Rosenbrock problem, formulated as the NLP:

minimize     x^2 + 100*z^2
subject to   z+(1-x)^2-y == 0

Joel Andersson, 2015
"""

# Declare variables
x = SX.sym("x")
y = SX.sym("y")
z = SX.sym("z")

# Formulate the NLP
f = x**2 + 100*z**2
g = z + (1-x)**2 - y
nlp = {'x':vertcat(x,y,z), 'f':f, 'g':g}

# Create an NLP solver
solver = nlpsol("solver", "ipopt", nlp)

# Solve the Rosenbrock problem
res = solver(x0  = [2.5,3.0,0.75],
             ubg = 0,
             lbg = 0)

# Print solution
print()
print("%50s " % "Optimal cost:", res["f"])
print("%50s " % "Primal solution:", res["x"])
print("%50s " % "Dual solution (simple bounds):", res["lam_x"])
print("%50s " % "Dual solution (nonlinear bounds):", res["lam_g"])



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs024():
    # The optimal objective is (if given in):
    f_opt = -1
    x_opt = cs.DM([3, cs.sqrt(3)])
    x = cs.MX.sym('x', 2)
    obj = cs.MX.zeros(1, 1)
    x0 = cs.DM.zeros(2, 1)
    lbx = -cs.inf*cs.DM.ones(2, 1)
    ubx = cs.inf*cs.DM.ones(2, 1)
    g = cs.MX.zeros(3, 1)
    lbg = -cs.inf*cs.DM.ones(3, 1)
    ubg = cs.inf*cs.DM.ones(3, 1)
    
    x0[0]= 1
    lbx[0]=0
    x0[1]= 1/2
    lbx[1]=0
    
    lbg[0] = 0
    g[0] = x[0]/cs.sqrt(3) - x[1]
    lbg[1] = 0
    g[1] = x[0] + cs.sqrt(3)*x[1]
    lbg[2] = -6
    g[2] = -x[0] - cs.sqrt(3)*x[1]
    obj = ((x[0] - 3)**2 - 9) * x[1]**3 / (27*cs.sqrt(3))
    
    
    f = obj
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
try:
  from lxml import etree
except:
  raise Exception("This functional relies on lxml.\nInstall it with apt-get install python-lxml.")

import sys
import re
import itertools
import json
import time

my_module = sys.argv[2]

t0 = time.time()

removed = 0
bag = dict()
# Parse the input XML
#e = etree.parse(sys.argv[1])
e = etree.iterparse(sys.argv[1],events=('end',))
for event, elem in e:
  if not elem.tag in bag:
    bag[elem.tag] = 1
  else:
    bag[elem.tag]+=1
  if elem.tag=='top':
    r = elem
  else:
    if elem.tag in ['typescopesitem']:
      removed+=1
      elem.clear()
      while elem.getprevious() is not None:
        del elem.getparent()[0]

print(bag)
print(removed)

#r = e.getroot()

def getAttribute(e,name,default=""):
  d = e.find('attributelist/attribute[@name="' + name + '"]')
  if d is None:
    return default
  return d.attrib["value"]

def getDocstring(e):
  return getAttribute(e,"feature_docstring")

def getModule(x):
  return x.xpath('ancestor-or-self::*/module/attributelist/attribute[@name="name"]')[-1].attrib['value']

def is_internal(d,msg=None):
  fd = d.find('attributelist/attribute[@name="feature_docstring"]')
  fn = d.find('attributelist/attribute[@name="name"]')
  name = [fn if fn is None else fn.attrib['value']]
  if fd is None:
    return False
    #raise Exception("is_internal find fail",d.tag,name,msg)
  v = fd.attrib['value']
  return v.strip().startswith("[INTERNAL]")
#  return "[INTERNAL]" in v
#  return not (("_EXPORT" in v) and ("CASADI_" in v))

print("elpased", time.time()-t0)
t0 = time.time()
print("get all the enums")
# get all the enums
enums = {}
for d in r.findall('*//enum'):
  if getModule(d) != my_module: continue
  sym_name = getAttribute(d,"sym_name")
  docs = getDocstring(d)
  #if is_internal(d):
  #  continue

  assert sym_name not in enums, "overwriting an enum"
  dt = enums[sym_name] = {"sym_name": sym_name, "docs": docs,"entries":{}}
  for e in d.findall('enumitem'):
    name = getAttribute(e,"name")
    ev = getAttribute(e,"enumvalueex")
    docs = getDocstring(d)
    ev = eval(ev,dict((k,v["ev"]) for k,v in dt["entries"].items()))

    assert name not in dt["entries"], "overwriting an enum entry"
    dt["entries"][name] = {"docs": docs, "ev": ev}


# get all the classes
internalClasses = []
classes0 = {}
symnameToName = {}
for c in r.findall('*//class'):
  name = c.find('attributelist/attribute[@name="name"]').attrib["value"]
  try:
    symname = c.find('attributelist/attribute[@name="sym_name"]').attrib["value"]
  except:
    continue
  if name == "casadi::"+symname:
    pass
  else:
    assert symname not in symnameToName, "overwriting a class symname"
    symnameToName[symname] = name
    assert "casadi::"+symname not in symnameToName, "overwriting a class symname"
    symnameToName["casadi::"+symname] = name

  classModule = c.find('attributelist/attribute[@name="module"]').attrib["value"]
  if name.startswith("std::vector<"): continue
  if name.startswith("std::pair<"): continue
  if classModule is None: raise ValueError("class module information missing")
  if classModule != getModule(c): raise ValueError("class module information different than getModule()")
  if classModule != my_module: continue
  if is_internal(c,msg="class"):
    internalClasses.append(name)
    continue

  assert name not in classes0, "overwriting a class"
  classes0[name] = c

internalClasses.extend(["casadi::SXNode",
                        "casadi::CallbackCPtr",
                        "casadi::DerivativeGeneratorCPtr",
                        "casadi::CustomEvaluateCPtr"])

def splitQualifiers(x):
  for q0 in ['r.','p.','q(const).']:
    if x.startswith(q0):
      (q1,ret) = splitQualifiers( x[len(q0):] )
      return ([q0]+q1, ret)
  return ([],x)

def stripQualifiers(x):
  return splitQualifiers(x[0])[1]

def internalClass(p):
  return stripQualifiers(p) in internalClasses

classes = {}
numInternalMethods = 0
numExposedMethods = 0
numExposedConstructors = 0
numInternalConstructors = 0

def getCanonicalType(x):
  (q,v) = splitQualifiers( x )
  if v in symnameToName:
    return ''.join(q)+symnameToName[v]
  else:
    return x

def getCanonicalParams(d,debug=""):
  params = []
  if d.find('attributelist/parmlist') is not None:
    for x in d.findall('attributelist/parmlist/parm/attributelist/attribute[@name="type"]'):
      name = x.findall('../attribute[@name="name"]')[0].attrib['value']
      params.append( (getCanonicalType(x.attrib['value']),"Output" if name=="OUTPUT" else "Normal") )

  return params
print("elpased", time.time()-t0)
t0 = time.time()
print("classes0")
for name,c in classes0.items():
  docs = getDocstring(c)
  if name in classes:
    data = classes[name]
  else:
    data = classes[name] = {'methods': [],"constructors":[],"docs": docs,"bases":[]}

  for d in c.findall('cdecl'):
    dname = d.find('attributelist/attribute[@name="name"]').attrib["value"]
    module = c.find('attributelist/attribute[@name="module"]').attrib["value"]
    if module != classModule:
      raise ValueError("method module",module,"!= class module",classModule)

    if (d.find('attributelist/attribute[@name="kind"]').attrib["value"]!="function"): continue
    if (d.find('attributelist/attribute[@name="kind"]').attrib["value"]!="function"): continue
    featureIgnore = d.find('attributelist/attribute[@name="feature_ignore"]')
    if featureIgnore is not None and featureIgnore.attrib['value'] == '1':
      continue

    params = getCanonicalParams(d,debug="method")

    if any([p[0].endswith("Creator") for p in params]): continue
    if any([internalClass(p[0]) for p in params]):
      numInternalMethods += 1
      continue

    rettype = getCanonicalType( d.find('attributelist/attribute[@name="type"]').attrib["value"] )

    if internalClass(rettype):
      numInternalMethods += 1
      continue
    storage = getAttribute(d,"storage")

    access = getAttribute(d,"access")
    if access in ["private", "protected"]: continue

    if dname == "ptr": continue # WORKAROUND FOR POTENTIAL SWIG BUG

    #if is_internal(d,msg="methods"):
    #  numInternalMethods += 1
    #  continue
    numExposedMethods += 1

    docs = getDocstring(d)

    data["methods"].append((dname,params,rettype,"Static" if storage=="static" else "Normal",docs))

  for d in itertools.chain(c.findall('constructor'),c.findall('extend/constructor')):
    params = getCanonicalParams(d,debug="constructor")
    if any([p[0].endswith("Creator") for p in params]): continue
    if any([internalClass(p) for p in params]):
      numInternalConstructors += 1
      continue

    rettype = name
    access = getAttribute(d,"access")
    if access in ["private", "protected"]: continue

    if is_internal(d,msg="constructors"):
      numInternalConstructors += 1
      continue
    if internalClass(rettype):
      numInternalConstructors += 1
      continue
    numExposedConstructors += 1
    docs = getDocstring(d)
    data["methods"].append(("CONSTRUCTOR",params,rettype,"Constructor",docs))

  for d in c.findall('attributelist/baselist/base'):
    base = d.attrib["name"]
    #if is_internal(d,msg="bases"): continue
    data["bases"].append( getCanonicalType( base ) )
print("elpased", time.time()-t0)
t0 = time.time()
print("classes")
for n,c in classes.items():
  new_bases = []
  for base in c['bases']:
    assert not base.startswith('std::'), "baseclass rename fail"
    if base.startswith('casadi::'):
      new_bases.append( getCanonicalType( base ) )
    else:
      new_bases.append( getCanonicalType( 'casadi::'+base ) )

  classes[n]['bases'] = new_bases


# get all the functions
functions = []
numInternalFunctions = 0
for d in r.findall('*//namespace/cdecl'):
  if d.find('attributelist/attribute[@name="sym_name"]') is None: continue
  if d.find('attributelist/attribute[@name="kind"]').attrib["value"]!="function": continue

  dname = d.find('attributelist/attribute[@name="sym_name"]').attrib["value"]
  name = d.find('attributelist/attribute[@name="name"]').attrib["value"]
  if dname == "dummy": continue
  code = ""
  friendwrap = "casadi_" in name

  if friendwrap:
    dname= "casadi_" + dname

  if my_module != getModule(d): continue
  if is_internal(d,msg="functions"):
    numInternalFunctions += 1
    continue

  params = getCanonicalParams(d,debug="function")

  if any([p.endswith("Creator") for p,_ in params]): continue
  if any([internalClass(p) for p,_ in params]):
    numInternalFunctions += 1
    continue

  rettype = getCanonicalType( getAttribute(d,"type") )
  if internalClass(rettype):
    numInternalFunctions += 1
    continue
  docs = getDocstring(d)

  functions.append({'funName':dname,
                    'funParams':params,
                    'funReturn':rettype,
                    'funDocs':"",#docs,
                    'funFriendwrap':friendwrap})



treedata = {"treeClasses": [],"treeFunctions": [], "treeEnums": {}}

code = sum([list(filter(lambda i: len(i.rstrip())> 0 ,x.attrib["value"].split("\n"))) for x in r.findall("*//insert/attributelist/attribute[@name='code']")],[])
treedata['treeIncludes'] = sorted(set(map(lambda x: x.rstrip(), list(filter(lambda y: re.search("^\s*#include ",y),code)))))

def getAllMethods(name,base=None):
  if base is None:
    base = name
  if name not in classes: return []
  c = classes[name]

  ret = c["methods"]
  return ret
  if name!=base:
    #ret = [(dname,params,base if mkind=="Constructor" else rettype,mkind) for (dname,params,rettype,mkind) in ret]
    # Omit baseclass constructors
    ret = list(filter(lambda x: x[3]!="Constructor", ret))

  for b in c["bases"]:
    ret = ret + getAllMethods(b,base)

  return ret
print("elpased", time.time()-t0)
t0 = time.time()
print("classes2")
myclasses = []
for k,v in classes.items():
  methods = []
  if "methods" in v:
    for (name,pars,rettype,mkind,docs) in getAllMethods(k): # v["methods"]:
      methods.append({"methodName": name, "methodReturn": rettype, "methodParams": pars, "methodKind": mkind,"methodDocs":""})

  treedata["treeClasses"].append({"classType": k, "classMethods": methods, "classDocs": v['docs']})
print("elpased", time.time()-t0)
t0 = time.time()
print("functions")
treedata["treeFunctions"] = functions
print("elpased", time.time()-t0)
t0 = time.time()
print("enums")
for k,v in enums.items():
  treedata["treeEnums"][k] = {
    "enumDocs": v['docs'],
    "enumEntries": dict(
       (kk , {"enumEntryDocs": vv["docs"],"enumEntryVal": vv["ev"]})
          for kk,vv in v["entries"].items())
  }
print("%5d classes %5d functions %5d enums" % (len(treedata['treeClasses']),
                                               len(treedata['treeFunctions']),
                                               len(treedata['treeEnums'])))

#print "classes:      %5d exposed %5d internal" % (len(classes),           len(internalClasses))
#print "methods:      %5d exposed %5d internal" % (numExposedMethods,      numInternalMethods)
#print "constructors: %5d exposed %5d internal" % (numExposedConstructors, numInternalConstructors)
#print "functions:    %5d exposed %5d internal" % (len(functions),         numInternalFunctions)

treedata["treeInheritance"] = dict((k, [i for i in v["bases"]]) for k,v in classes.items())
print("elpased", time.time()-t0)
t0 = time.time()

print("dump")
with open(my_module+'.json', 'w') as outfile:
    json.dump(treedata, outfile, indent=True)
print("elpased", time.time()-t0)
t0 = time.time()



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
from numpy import Inf

def reportBounds(value,lowerbound,upperbound,labels=None,tol=1e-8,showNonViolating=True):
  if hasattr(labels,"labels"):
    labels = labels.labels()

  v = list(value.nonzeros())
  lb = list(lowerbound.nonzeros())
  ub = list(upperbound.nonzeros())

  if labels is None:
    labels = [""] * len(v) 
  
  if not(len(v)==len(lb) and len(lb)==len(ub)):
    raise Exception("value, lowerbound and upperbound must all be the same size, but got %d, %d and %d. " % (len(v),len(lb),len(ub)))
  
  if len(labels)!=len(v):
    raise Exception("Labels (%d) must be same size as values (%d)" % (len(labels),len(v)))
  
  if ( all(value <= upperbound + tol) and all(value >= lowerbound - tol) ):
    print("All %d bounds are met: " % value.size())
  else:
    print("Problem with bounds : ")
  
  print("-"*60)

  # The length of the numeric fields
  fieldlength = 10
  # The length of the constraint visualizer strip
  indicator_length = 15
  

      
  # Loop over the elements of value
  for i in range(value.size()):
    violated =  (v[i] > (ub[i] + tol) or v[i] < (lb[i] - tol))
    nonregular = not is_regular([v[i]])

    if not(showNonViolating) and not(violated) and not(nonregular):
      continue
      
    if labels is None:
      identifier = "%d." % i
    else:
      identifier = labels[i]
         
   
      
    if ( abs(lb[i] - ub[i])<=tol):
      midfield = "%*s == %*s " % (fieldlength, "%.7e" % lb[i], fieldlength, "%.7e" % v[i])
      indicator = ""
    else:
      indicator = "-" * indicator_length
      if lb[i]==Inf:
        indicator = "8" +  indicator
      elif (abs(v[i]-lb[i])<=tol):
        indicator = "X" +  indicator
      else:
        indicator = "o" +  indicator
      if ub[i]==Inf:
        indicator += "8"
      elif (abs(v[i]-ub[i])<=tol):
        indicator += "X"
      else:
        indicator += "o"

      if (v[i] <= (ub[i] + tol) and v[i] >= (lb[i] - tol)):
        index = (v[i]-lb[i])/(ub[i]-lb[i])*(indicator_length-1)
        index = min(max(0,index),indicator_length-1)
        index = int(index)
        indicator = indicator[:1+index] + '=' + indicator[1+index:]
      
      midfield = "%*s <= %*s <= %*s" % (fieldlength, "%.7e" % lb[i], fieldlength, "%.7e" % v[i], fieldlength, "%.7e" % ub[i])

    if (v[i] > (ub[i] + tol) or v[i] < (lb[i] - tol)):
      indicator = " VIOLATED "
    if nonregular:
      indicator = " !REGULAR "
  
    print("%15s | %*s | %*s" % (identifier, (fieldlength + 6) * 3 , midfield, indicator_length+3, indicator))
  
  print("-" * 60)
  



import subprocess
import re
import os

targets = set()
for L in file("src/Makefile","r"):
  m = re.match("ifeq \(\$\(TARGET\), ([A-Z0-9_]+)\)", L)
  if m:
    targets.add(m.group(1))

targets = sorted(list(targets))

combinations = [
("X64_AVX","GENERIC"),
("X64_AVX2","X64_INTEL_HASWELL"),
("X64_AVX","X64_INTEL_SANDY_BRIDGE"),
("X64_SSE3","X64_INTEL_CORE"),
("X64_SSE3","X64_AMD_BULLDOZER"),
("C99_4X4","GENERIC")]
#("C99_4X4_PREFETCH","GENERIC")]

#("CORTEX_A7","GENERIC"),
#("CORTEX_A9","GENERIC"),
#("CORTEX_A15","GENERIC"),
#("CORTEX_A57","GENERIC"),
#("C99_4X4","GENERIC"),
#("C99_4X4_PREFETCH","GENERIC")
#]

target_data = {}

this_pwd = os.getcwd()
for t in targets:
  p = subprocess.Popen(["make","CC=echo compiler_action `pwd`","TARGET=%s" % t],stdout=subprocess.PIPE,stderr=subprocess.PIPE,cwd="src")

  stdout,stderr = p.communicate()

  data = []

  for L in stdout.split("\n"):
    if L.startswith("compiler_action"):
      line = L.split(" ")
      pwd = line[1][len(this_pwd)+1:]
      cfile = line[-1]
      flags = [f for f in line[2:-4] if not f.startswith("-I") and not f.startswith("-DOS")]
      data.append((cfile,pwd,flags))

  for cfile,pwd,f in data:
    assert f==data[0][2] or len(f)==0
  target_data[t] = data

with file('CMakeLists.txt','w') as f:
  f.write("cmake_minimum_required(VERSION 3.10.2)\n")
  f.write("""
    if(WIN32)
    set(HPMPC_FLAGS -DOS_WINDOWS)
    elseif(APPLE)
     set(HPMPC_FLAGS -DOS_MAC)
    else()
     set(HPMPC_FLAGS -DOS_LINUX)
    endif()
    if()
    set(CMAKE_SHARED_LINKER_FLAGS ${CMAKE_SHARED_LINKER_FLAGS} -Wl,--unresolved-symbols=report-all)
    endif()
    """)
  f.write("include_directories(${CMAKE_CURRENT_SOURCE_DIR}/../blasfeo/src/include)\n")
  f.write("include_directories(${CMAKE_CURRENT_SOURCE_DIR}/dummy_include)\n")
  f.write("include_directories(${CMAKE_CURRENT_SOURCE_DIR}/dummy_include/foo/bar)\n")
  #f.write("include_directories(/opt/blasfeo/include)\n")

  for t,blasfeo_target in combinations:
    data = target_data[t]

    libname = "casadi_hpmpc_%s_%s" % (t,blasfeo_target)
    blasfeo_libname = "casadi_blasfeo_%s" % blasfeo_target
    bitness64 = "-m64" in data[0][2]
    if bitness64:
      f.write("if( CMAKE_SIZEOF_VOID_P EQUAL 8 )\n")
    f.write("add_library(%s SHARED %s)\n" % (libname, "\n".join([ os.path.join(pwd,cfile) for cfile, pwd, _ in data])))
    f.write("SET_TARGET_PROPERTIES(%s PROPERTIES COMPILE_FLAGS \"-DLA_BLASFEO -DTARGET_%s -DTARGET_%s %s ${HPMPC_FLAGS}\")\n\n" % (libname, blasfeo_target,t , " ".join(data[0][2])))
    f.write("target_link_libraries(%s %s m)\n" % (libname, blasfeo_libname))
    if bitness64:
      f.write("endif()\n")



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA


from casadi import *
import numpy

# Let's construct a block diagonal structure

b1 = DM([[2,3],[4,5]])
b2 = DM([[6,7,8],[9,10,11],[12,13,14]])
A = diagcat(1,b1,b2,15)

print("original: ")
print(A)

# Ruin the nice structure

numpy.random.seed(0)
p1 = numpy.random.permutation(A.size1())
p2 = numpy.random.permutation(A.size2())

S = A[p1,:]
#S = A[p1,p2]

print("randomly permuted: ")
print(S)
nb, rowperm, colperm, rowblock, colblock, coarse_rowblock, coarse_colblock = S.sparsity().btf()

print("number of blocks: ", nb)
print("rowperm: ", rowperm)
print("colperm: ", colperm)
print("restored:")
print(S[rowperm,colperm])
print("rowblock: ", rowblock)
print("colblock: ", colblock)
print("coarse_rowblock: ", coarse_rowblock)
print("coarse_colblock: ", coarse_colblock)




#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
import unittest
from types import *
from helpers import *
import random

warnings.filterwarnings("ignore",category=DeprecationWarning)

lsolvers = []
try:
  load_linsol("csparse")
  lsolvers.append(("csparse",{},set()))
except:
  pass

"""
try:
  load_linsol("csparsecholesky")
  lsolvers.append(("csparsecholesky",{},{"posdef","symmetry"}))
except:
  pass
"""
try:
  load_linsol("lapacklu")
  lsolvers.append(("lapacklu",{},set()))
except:
  pass

try:
  load_linsol("lapackqr")
  lsolvers.append(("lapackqr",{},set()))
except:
  pass

try:
  load_linsol("symbolicqr")
  lsolvers.append(("symbolicqr",{},set()))
except:
  pass

try:
  load_linsol("lsqr")
  lsolvers.append(("lsqr",{},set()))
except:
  pass

try:
  load_linsol("ma27")
  lsolvers.append(("ma27",{},{"symmetry"}))
except:
  pass

try:
  load_linsol("mumps")
  lsolvers.append(("mumps",{},{"symmetry"}))
except:
  pass

try:
  load_linsol("qr")
  lsolvers.append(("qr",{},set()))
except:
  pass

try:
  load_linsol("ldl")
  lsolvers.append(("ldl",{},{"posdef","symmetry"}))
except:
  pass


nsolvers = []

def nullspacewrapper(name, sp, options):
  a = SX.sym("a",sp)
  f = Function(name, [a],[nullspace(a)],options)
  return f

nsolvers.append((nullspacewrapper,{},set()))

print("linear solvers", lsolvers)

class LinearSolverTests(casadiTestCase):

  def test_nullspace(self):

    for A in  [
                  DM([[1,1.3],[2,5],[1,0.5],[1.8,1.7]]),
                  DM([[1,1.3],[2,5],[1,0.5]]),
                  DM([[1,1.3],[2,5],[1,0.5],[0.2,0.3],[-0.3,0.7]]),
                  DM([[1,0],[0,0],[0,1],[0,0]]),
                  DM([[1.3,0,0.4,1],[0.2,0.1,11,0],[0,1,0,0],[0.7,0.9,0,0],[1.1,0.99,0,0]])
              ]:
      n ,m = A.shape
      for Solver, options,req in nsolvers:
        if "symmetry" in req: continue
        solver = Solver("solver", A.T.sparsity(), options)
        solver_in = [0]*solver.n_in();solver_in[0]=A.T

        solver_out = solver.call(solver_in)

        self.checkarray(mtimes(A.T,solver_out[0]),DM.zeros(m,n-m))
        self.checkarray(mtimes(solver_out[0].T,solver_out[0]),DM.eye(n-m))

        options["ad_weight"] = 0
        options["ad_weight_sp"] = 0
        solver = Solver("solver", A.T.sparsity(), options)

        Jf = jacobian_old(solver, 0, 0)

        options["ad_weight"] = 1
        options["ad_weight_sp"] = 1
        solver = Solver("solver", A.T.sparsity(), options)

        Jb = jacobian_old(solver, 0, 0)

        Jf_in = [0]*Jf.n_in();Jf_in[0]=A.T
        Jb_in = [0]*Jb.n_in();Jb_in[0]=A.T

        Jf_out = Jf.call(Jf_in)
        Jb_out = Jb.call(Jb_in)

        self.checkarray(Jf_out[0],Jb_out[0])
        self.checkarray(Jf_out[1],Jb_out[1])

        d = solver.forward(1)

        r = numpy.random.rand(*A.shape)

        d_in = [A.T, 5, r.T]

        d_out = d.call(d_in)

        exact = d_out[0]

        solver_in = [0]*solver.n_in();solver_in[0]=A.T
        solver_out = solver(*solver_in)
        nom = solver_out

        eps = 1e-6
        solver_in = [0]*solver.n_in();solver_in[0]=(A+eps*r).T
        solver_out = solver(*solver_in)
        pert = solver_out

        fd = (pert-nom)/eps

        #print exact, fd

        #print numpy.linalg.svd(horzcat(*[exact, fd]).T)[1]

        #print "fd:", mtimes(fd.T,fd), numpy.linalg.eig(mtimes(fd.T,fd))[0]
        #print "exact:", mtimes(exact.T,exact), numpy.linalg.eig(mtimes(exact.T,exact))[0]
        #print "fd:", mtimes(fd,fd.T), numpy.linalg.eig(mtimes(fd,fd.T))[0]
        #print "exact:", mtimes(exact,exact.T), numpy.linalg.eig(mtimes(exact,exact.T))[0]

        V = numpy.random.rand(A.shape[0]-A.shape[1],A.shape[0]-A.shape[1])
        V = V+V.T
        print(V)
        #V = DM.eye(A.shape[0]-A.shape[1])
        a = mtimes([nom,V,fd.T])+mtimes([fd,V,nom.T])
        b = mtimes([nom,V,exact.T])+mtimes([exact,V,nom.T])

        print("here:", a-b)

        #self.checkarray(a,b,digits=5)

        V = numpy.random.rand(A.shape[0],A.shape[0])
        V = V+V.T
        V = DM.eye(A.shape[0])
        a = mtimes([nom.T,V,fd])+mtimes([fd.T,V,nom])
        b = mtimes([nom.T,V,exact])+mtimes([exact.T,V,nom])

        self.checkarray(a,b,digits=5)

  def test_simple_solve(self):
    A_ = DM([[3,7],[1,2]])
    b_ = DM([1,0.5])

    A = MX.sym("A",A_.sparsity())
    b = MX.sym("b",b_.sparsity())

    for Solver, options,req in lsolvers:
      C = solve(A,b,Solver,options)
      if "symmetry" in req:
        A0 = A_.T+A_
      else:
        A0 = A_
      f = Function("f", [A,b],[C])
      f_out = f(A0, b_)

      self.checkarray(f_out,np.linalg.solve(A0,b_))
      self.checkarray(mtimes(A0,f_out),b_)

  def test_pseudo_inverse(self):
    numpy.random.seed(0)
    A_ = DM(numpy.random.rand(4,6))

    A = MX.sym("A",A_.sparsity())
    As = SX.sym("A",A_.sparsity())

    for Solver, options,req in lsolvers:
      if "symmetry" in req: continue
      print(Solver)
      B = pinv(A,Solver,options)

      f = Function("f", [A],[B])
      f_out = f(A_)

      self.checkarray(mtimes(A_,f_out),DM.eye(4))

      f = Function("f", [As],[pinv(As)])
      f_out = f(A_)

      self.checkarray(mtimes(A_,f_out),DM.eye(4))

      solve(mtimes(A,A.T),A,Solver,options)
      pinv(A_,Solver,options)

      #self.checkarray(mtimes(A_,pinv(A_,Solver,options)),DM.eye(4))

    A_ = DM(numpy.random.rand(3,5))

    A = MX.sym("A",A_.sparsity())
    As = SX.sym("A",A_.sparsity())

    for Solver, options,req in lsolvers:
      print(Solver)
      if "symmetry" in req: continue
      B = pinv(A,Solver,options)

      f = Function("f", [A],[B])
      f_out = f(A_)
      self.checkarray(mtimes(A_,f_out),DM.eye(3))

      f = Function("f", [As],[pinv(As)])
      f_out = f(A_)

      self.checkarray(mtimes(A_,f_out),DM.eye(3))

      #self.checkarray(mtimes(pinv(A_,Solver,options),A_),DM.eye(3))

  def test_simple_solve_dmatrix(self):
    A = DM([[3,7],[1,2]])
    b = DM([1,0.5])
    for Solver, options, req in lsolvers:
      print(Solver)
      if "symmetry" in req:
        A0 = A.T+A
      else:
        A0 = A
      C = solve(A0,b,Solver,options)

      sol = np.linalg.solve(A0,b)
      self.checkarray(C,sol)
      self.checkarray(mtimes(A0,sol),b)

  def test_simple_trans(self):
    A = DM([[3,1],[7,2]])
    for Solver, options,req in lsolvers:
      if "symmetry" in req:
        A0 = A.T+A
      else:
        A0 = A
      solver = casadi.Linsol("solver", Solver, A0.sparsity(), options)
      b = DM([1,0.5])
      x = solver.solve(A0.T, b)
      res = np.linalg.solve(A0.T,b)
      self.checkarray(x, res)

  def test_simple(self):
    A = DM([[3,1],[7,2]])
    for Solver, options, req in lsolvers:
      print(Solver)
      if "symmetry" in req:
        A0 = A.T+A
      else:
        A0 = A
      solver = casadi.Linsol("solver", Solver, A0.sparsity(), options)
      b = DM([1,0.5])
      x = solver.solve(A0, b)
      res = np.linalg.solve(A0,b)
      self.checkarray(x, res)

  def test_simple_function_indirect(self):

    for Solver, options,req in lsolvers:
      A_ = DM([[3,1],[7,2]])
      A = MX.sym("A",A_.sparsity())
      b_ = DM([1,0.5])
      b = MX.sym("b",b_.sparsity())
      if "symmetry" in req: A_ = A_.T + A_
      print(Solver)
      solver = casadi.Linsol("solver", Solver, A.sparsity(), options)
      solver_in = {}
      solver_in["A"]=A_
      solver_in["B"]=b_

      sol = {"X": solver.solve(A,b)}
      sol["A"] = A
      sol["B"] = b
      relay = Function("relay", sol, ["A","B"], ["X"])

      A_0 = A[0,0]
      A_1 = A[0,1]
      A_2 = A[1,0]
      A_3 = A[1,1]

      b_0 = b[0]
      b_1 = b[1]

      solution = Function("solution", {"A":A, "B":b, "X":vertcat(*[(((A_3/((A_0*A_3)-(A_2*A_1)))*b_0)+(((-A_1)/((A_0*A_3)-(A_2*A_1)))*b_1)),((((-A_2)/((A_0*A_3)-(A_2*A_1)))*b_0)+((A_0/((A_0*A_3)-(A_2*A_1)))*b_1))])}, ["A", "B"], ["X"])


      self.checkfunction(relay,solution,inputs=solver_in)
      self.check_serialize(relay,inputs=solver_in)

      if Solver in ["qr","ldl"]:
        self.check_codegen(relay,inputs=solver_in)

  @memory_heavy()
  def test_simple_solve_node(self):

    for Solver, options, req in lsolvers:

      for A_,b_ in [
                       (DM([[3,1],[7,2]]),DM([[1,0.3],[0.5,0.7]])),
                       (sparsify(DM([[3,0],[7,2]])),DM([[1,0.3],[0.5,0.7]])),
                       (DM([[3,1],[7,2]]),sparsify(DM([[1,0],[0,0.7]])))
                   ]:
        if "symmetry" in req: A_ = A_.T+A_

        A = MX.sym("A",A_.sparsity())
        b = MX.sym("b",b_.sparsity())
        print(Solver)
        solver = casadi.Linsol("solver", Solver, A.sparsity(), options)
        for tr in [True, False]:
          x = solver.solve(A,b,tr)
          f = Function("f", [A,b],[x])
          f_out = f(A_, b_)

          if tr:
            A_0 = A[0,0]
            A_1 = A[1,0]
            A_2 = A[0,1]
            A_3 = A[1,1]
          else:
            A_0 = A[0,0]
            A_1 = A[0,1]
            A_2 = A[1,0]
            A_3 = A[1,1]

          b_0 = b[0,0]
          b_1 = b[1,0]

          c_0 = b[0,1]
          c_1 = b[1,1]

          solution = Function("solution", [A,b],[blockcat([[(((A_3/((A_0*A_3)-(A_2*A_1)))*b_0)+(((-A_1)/((A_0*A_3)-(A_2*A_1)))*b_1)),(((A_3/((A_0*A_3)-(A_2*A_1)))*c_0)+(((-A_1)/((A_0*A_3)-(A_2*A_1)))*c_1))],[((((-A_2)/((A_0*A_3)-(A_2*A_1)))*b_0)+((A_0/((A_0*A_3)-(A_2*A_1)))*b_1)),((((-A_2)/((A_0*A_3)-(A_2*A_1)))*c_0)+((A_0/((A_0*A_3)-(A_2*A_1)))*c_1))]])])

          solution_in = [0]*solution.n_in();solution_in[0]=A_
          solution_in[1]=b_

          self.checkfunction(f,solution,inputs=solution_in)

          if "SymbolicQR" not in str(Solver) : continue
          solversx = f.expand('expand_'+f.name())
          solversx_in = [0]*solversx.n_in();solversx_in[0]=A_
          solversx_in[1]=b_

          self.checkfunction(solversx,solution,digits_sens = 7)

  def test_large_sparse(self):

    n = 10

    for Solver, options,req in lsolvers:
      numpy.random.seed(1)
      A = self.randDM(n,n,sparsity=0.5)
      if "symmetry" in req: A = A.T+A
      b = self.randDM(n,3,sparsity=0.5)

      As = MX.sym("A",A.sparsity())
      bs = MX.sym("B",b.sparsity())

      C = solve(A,b,Solver,options)

      self.checkarray(mtimes(A,C),b)

      f = Function("f", [As,bs],[solve(As,bs,Solver,options)])
      f_out = f(A, b)

      self.checkarray(mtimes(A,f_out),b)

  def test_ma27(self):
      n = np.nan

      A = DM([[2, 3, n, n, n],
             [3, 0, 4, n, 6],
             [n, 4, 1, 5, n],
             [n, n, 5, 0, n],
             [n, 6, n, n, 1]])
      A = A[sparsify(A==A).sparsity()]

      b = DM([8,45,31,15,17])

      ref = np.linalg.solve(A,b)
      for Solver, options, req in lsolvers:
        if "posdef" in req: continue
        As = MX.sym("A",A.sparsity())
        bs = MX.sym("B",b.sparsity())
        C = solve(A,b,Solver,options)
        self.checkarray(ref,C)


  def test_large_sparse2(self):
    numpy.random.seed(1)
    n = 10

    for Solver, options, req in lsolvers:

      print(Solver)
      A = self.randDM(n,n,sparsity=0.5)
      b = self.randDM(n,3)
      if "posdef" in req:
        A = mtimes(A.T, A)
        A = densify(A)
      elif "symmetry" in req:
        A = A.T+A
        A[Sparsity.diag(n)] =1e-8
        A = densify(A)

      As = MX.sym("A",A.sparsity())
      bs = MX.sym("B",b.sparsity())
      C = solve(A,b,Solver,options)
      digits = 7 if "ma" in str(Solver) else 10

      self.checkarray(mtimes(A,C),b,digits=digits)

      for As_,A_ in [(As,A),(densify(As),densify(A)),(densify(As).T,densify(A).T),(densify(As.T),densify(A.T)),(As.T,A.T)]:
        f = Function("f", [As,bs],[solve(As_,bs,Solver,options)])
        f_out = f(A, b)

        self.checkarray(mtimes(A_,f_out),b,digits=digits)

  def test_dimmismatch(self):
    A = DM.eye(5)
    b = DM.ones((4,1))
    As = MX.sym("A",A.sparsity())
    bs = MX.sym("b",b.sparsity())

    for Solver, options, req in lsolvers:
      with self.assertRaises(Exception):
        solve(A,b,Solver,options)

      with self.assertRaises(Exception):
        solve(As,bs,Solver,options)

  def test_cache(self):
    n = 5

    As = [np.random.random((n,n)) for i in range(10)]

    A = MX.sym("A",n,n)
    f = Function('f',[A],[solve(A, DM.ones(n), "qr", {"cache": 2})])

    S1 = f(As[0])
    r = f(As[0])
    self.checkarray(S1, r)

    S2 = f(As[1])
    r = f(As[1])
    self.checkarray(S2, r)
    r = f(As[0])
    self.checkarray(S1, r)

    S3 = f(As[2])
    r = f(As[2])
    self.checkarray(S3, r)
    r = f(As[0])
    self.checkarray(S1, r)
    r = f(As[1])
    self.checkarray(S2, r)

    self.check_codegen(f, inputs=[As[0]])
    self.check_serialize(f, inputs=[As[0]])

  @memory_heavy()
  def test_thread_safety(self):
    x = MX.sym('x')
    y = MX.sym('y')


    for Solver, options, req in lsolvers:
      if "mumps" in str(Solver): continue
      print(Solver)
      f = Function('f', [x, y], [x ** 2 - y])
      finv = rootfinder('finv', "newton", f, {"linear_solver": Solver, "linear_solver_options": options})

      finv_par = finv.map(200, 'thread',4)
      res = finv_par(numpy.ones(200), numpy.linspace(0, 10, 200))
      self.checkarray(norm_inf(res.T-sqrt(numpy.linspace(0, 10, 200))),0, digits=5)

      z = solve(blockcat([[1+x,0],[0,1+y]]),vertcat(x,y), Solver, options)
      f = Function('f',[x,y],[z])
      f_par = f.map(200, 'thread',4)
      res = f_par(numpy.linspace(10, 0, 200), numpy.linspace(0, 10, 200))


  def test_issue2664(self):

    bnum = DM.rand(3,3)

    x = MX.sym('x',3,3)
    for b in [MX.sym('b',3,3),MX.sym('b',Sparsity.lower(3)),MX.sym('b',Sparsity.upper(3))]:
      f = Function('f',[x,b],[solve(x,b,'symbolicqr')])

      for bn in [bnum,bnum[Sparsity.lower(3)],bnum[Sparsity.upper(3)]]:

        A = evalf(f(diagcat(1,2,3),SX(bn)))
        B = f(diagcat(1,2,3),bn)
        self.checkarray(A,B)

  @memory_heavy()
  def test_issue3489(self):

    import casadi as ca

    E_f = ca.SX.sym("E_f")
    E_r = ca.SX.sym("E_r")
    T = ca.SX.sym("T")
    k_f = ca.SX.sym("k_f")
    k_r = ca.SX.sym("k_r")

    k0_f = ca.SX.sym("k0_f")
    k0_r = ca.SX.sym("k0_r")
    x_1 = ca.SX.sym("x_1")
    x_2 = ca.SX.sym("x_2")
    x_3 = ca.SX.sym("x_3")
    x_4 = ca.SX.sym("x_4")
    r_1 = ca.SX.sym("r_1")
    r_2 = ca.SX.sym("r_2")
    r_3 = ca.SX.sym("r_3")
    r_4 = ca.SX.sym("r_4")

    y_1 = ca.SX.sym("y_1")
    y_2 = ca.SX.sym("y_2")
    y_3 = ca.SX.sym("y_3")
    y_4 = ca.SX.sym("y_4")


    eq1 = k_f - k0_f -E_f + T
    eq2 = k_r - k0_r -E_r + T
    eq3 = r_1 - k_f + (-x_1) + (-x_2) - k_r + (x_3) + (x_4)
    eq4 = r_2 - k_f + (-x_1) + (-x_2) - k_r + (x_3) + (x_4)
    eq5 = r_3 - k_f + (-x_1) + (-x_2) - k_r + (x_3) + (x_4)
    eq6 = r_4 - k_f + (-x_1) + (-x_2) - k_r + (x_3) + (x_4)
    eq7 = 1 * 0.2 - 1 * x_1 + r_1
    eq8 = 1 * 0.8 - 1 * x_2 + r_2
    eq9 = -1 * x_3 + r_3
    eq10 = -1 * x_4 + r_4

    eq11 = x_1 - y_1
    eq12 = x_2 - y_2
    eq13 = x_3 - y_3
    eq14 = x_4 - y_4

    all_vars = [k_f, k_r, x_1, x_2, x_3, x_4, r_1, r_2, r_3, r_4]
    all_vars.extend([y_1, y_2, y_3, y_4])


    all_eq = []

    all_eq.extend([eq11, eq12, eq13, eq14])
    all_eq.extend([eq3, eq4, eq5, eq6, eq7, eq8, eq9, eq10])
    all_eq.extend([eq1, eq2,])

    x = ca.vcat(all_vars)
    p = ca.vcat([E_f, E_r, k0_f, k0_r, T])
    g = ca.vcat(all_eq)

    A = evalf(jacobian(g,x))
    B = evalf(jacobian(g,p))

    Amx = MX.sym("A",A.sparsity())
    Bmx = MX.sym("B",B.sparsity())

    rf = {"x": x, "p": p, "g": g}

    sim = ca.rootfinder("s", "fast_newton", rf, {"ad_weight_sp":0})
    S1 = sim.jac_sparsity(0,1)
    sim = ca.rootfinder("s", "fast_newton", rf, {"ad_weight_sp":1})
    S2 = sim.jac_sparsity(0,1)
    
    self.assertTrue(S1==S2)
    
    np.random.seed(1)
    m = 14
    p = 13
    for i in range(25):
        while True:
            A = self.randDM(m,m,sparsity=0.15)
            if sprank(A)==m:
              break
        B = self.randDM(m,p,sparsity=0.1)
        C = sparsify(solve(A,B),1e-7).sparsity()
    
        P = MX.sym("p", p)

        f0 = Function("f",[P],[solve(A, mtimes(B,P))], {"ad_weight_sp": 0})
        S1=f0.jac_sparsity(0,0)
 
        f1 = Function("f",[P],[solve(A, mtimes(B,P))], {"ad_weight_sp": 1})
        S2=f1.jac_sparsity(0,0)
      
        self.assertTrue(S1==S2)
        self.assertTrue(C.is_subset(S1))
    

if __name__ == '__main__':
    unittest.main()



#
#     This file is part of CasADi.
#
#     CasADi -- A symbolic framework for dynamic optimization.
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl,
#                             KU Leuven. All rights reserved.
#     Copyright (C) 2011-2014 Greg Horn
#
#     CasADi is free software; you can redistribute it and/or
#     modify it under the terms of the GNU Lesser General Public
#     License as published by the Free Software Foundation; either
#     version 3 of the License, or (at your option) any later version.
#
#     CasADi is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#     Lesser General Public License for more details.
#
#     You should have received a copy of the GNU Lesser General Public
#     License along with CasADi; if not, write to the Free Software
#     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
#
from casadi import *
import casadi as c
import numpy
import unittest
from types import *
from helpers import *
import numpy
import random

class Sparsitytests(casadiTestCase):
  def test_union(self):
    self.message("Sparsity union")
    nza = set([  (0,0),
             (0,1),
             (2,0),
             (3,1)])
    nzb = set([  (0,2),
             (0,0),
             (2,2)])

    a = Sparsity(4,5)
    for i in nza:
      a.add_nz(i[0],i[1])

    b = Sparsity(4,5)
    for i in nzb:
      b.add_nz(i[0],i[1])

    c =a.unite(b)

    c = a + b
    self.assertEqual(c.nnz(),len(nza.union(nzb)))
    for k in range(c.nnz()):
      ind = (c.row(k),c.get_col()[k])
      self.assertTrue(ind in nza or ind in nzb)

  def test_intersection(self):
    self.message("Sparsity intersection")
    nza = set([  (0,0),
             (0,1),
             (2,0),
             (3,1),
             (2,3)])
    nzb = set([  (0,2),
             (0,0),
             (2,2),
             (2,3)])

    a = Sparsity(4,5)
    for i in nza:
      a.add_nz(i[0],i[1])

    b = Sparsity(4,5)
    for i in nzb:
      b.add_nz(i[0],i[1])

    c=a.intersect(b)
    for k in range(c.nnz()):
      ind = (c.row(k),c.get_col()[k])
      self.assertTrue(ind in nza and ind in nzb)

    c = a * b
    self.assertEqual(c.nnz(),len(nza.intersection(nzb)))
    for k in range(c.nnz()):
      ind = (c.row(k),c.get_col()[k])
      self.assertTrue(ind in nza and ind in nzb)

  def test_get_nzDense(self):
    self.message("get_nzDense")
    nza = set([  (0,0),(0,1),(2,0),(3,1)])

    a = Sparsity(4,5)
    for i in nza:
      a.add_nz(i[0],i[1])

    A = DM.ones(a)
    Ad = DM(array(A))
    for i in a.find():
      self.assertEqual(Ad.nz[i],1)

  def test_find_nonzero(self):
    numpy.random.seed(0)
    d = self.randDM(20,10,0.6)
    sp = d.sparsity()

    sp2 = Sparsity.nonzeros(20,10,sp.find())
    self.assertTrue(sp==sp2)

  def test_enlarge(self):
    self.message("enlarge")
    import numpy
    self.message(":dense")
    #sp = Sparsity(3,4,[1,2,1],[0,2,2,3])
    sp = Sparsity.dense(3,4)

    col = [1,2,4]
    row = [0,3,4,6]
    sp.enlarge(7,8,col,row)

    z = numpy.zeros((7,8))
    for i in col:
      for j in row:
        z[i,j]=1

    self.checkarray(DM.ones(sp),z,"enlarge")
    self.message(":sparse")
    sp = Sparsity(4,3,[0,2,2,3],[1,2,1]).T
    n = DM.ones(sp)
    z = numpy.zeros((7,8))
    for i in range(3):
      for j in range(4):
          z[col[i],row[j]]= n[i,j]
    sp.enlarge(7,8,[1,2,4],[0,3,4,6])

    self.checkarray(DM.ones(sp),z,"enlarge")

  def tomatrix(self,s):
    d = DM.ones(s)
    for k in range(d.nnz()):
      d.nz[k] = k+1
    return d

  def test_NZ(self):
    self.message("NZ constructor")
    nza = [  (0,0),
             (0,1),
             (2,0),
             (2,3),
             (2,4),
             (3,1)]

    a = Sparsity(4,5)
    for i in nza:
      a.add_nz(i[0],i[1])

    b = Sparsity.triplet(4,5,[i[0] for i in nza],[i[1] for i in nza])
    self.checkarray(self.tomatrix(a),self.tomatrix(b),"rowcol")

  def test_rowcol(self):
    self.message("rowcol constructor")

    r = [0,1,3]
    c = [1,4]
    a = Sparsity(4,5)
    for i in r:
      for j in c:
        a.add_nz(i,j)

    b = Sparsity.rowcol(r,c,4,5)
    self.checkarray(self.tomatrix(a),self.tomatrix(b),"rowcol")

  def test_reshape(self):
    self.message("Reshape")
    nza = set([  (0,0),
             (0,1),
             (2,0),
             (2,3),
             (2,4),
             (3,1)])

    a = Sparsity(4,5)
    for i in nza:
      a.add_nz(i[0],i[1])

    A=self.tomatrix(a).full()
    B=self.tomatrix(casadi.reshape(a,2,10)).full()
    B_=numpy.reshape(A.T,(10,2)).T

    self.checkarray(B,B_,"reshape")

  def test_vec(self):
    return # This test doesn't make much sense
    self.message("vec")
    nza = set([  (0,0),
             (0,1),
             (2,0),
             (2,3),
             (2,4),
             (3,1)])

    a = Sparsity(4,5)
    for i in nza:
      a.add_nz(i[0],i[1])

    A=self.tomatrix(a).full()
    B=self.tomatrix(vec(a)).full()
    B_=numpy.reshape(A,(20,1))

    self.checkarray(B,B_,"reshape")


  def test_refcount(self):
      x = DM(Sparsity.lower(4),5)
      s = mtimes(x,x).sparsity()
      self.assertEqual(s.numel(),16)

  def test_splower(self):
    sp = Sparsity(4,3,[0,2,2,3],[1,2,1])
    print(array(sp))
    print(array(tril(sp)))
    print(sp.get_lower())


  def test_diag(self):
    self.message("diag")
    A = Sparsity(5,5)
    A.add_nz(1,1)
    A.add_nz(2,4)
    A.add_nz(3,3)

    sp, mapping = A.get_diag()
    B = DM.ones(sp)

    self.checkarray(array([[0],[1],[0],[1],[0]]),B,"get_diag(matrix)")
    self.checkarray(array([0,1]),array(list(mapping)),"get_diag(vector)")

    #print B

    A = Sparsity(5,1)
    A.add_nz(1,0)
    A.add_nz(2,0)
    A.add_nz(4,0)

    sp, mapping = A.get_diag()
    B = DM.ones(sp)

    self.checkarray(array([[0,0,0,0,0],[0,1,0,0,0],[0,0,1,0,0],[0,0,0,0,0],[0,0,0,0,1]]),B,"get_diag(vector)")

    self.checkarray(array([0,1,2]),array(list(mapping)),"get_diag(vector)")

    A = Sparsity(1,5)
    A.add_nz(0,1)
    A.add_nz(0,2)
    A.add_nz(0,4)

    sp, mapping = A.get_diag()
    B = DM.ones(sp)

    self.checkarray(array([[0,0,0,0,0],[0,1,0,0,0],[0,0,1,0,0],[0,0,0,0,0],[0,0,0,0,1]]),B,"get_diag(vector)")

    self.checkarray(array([0,1,2]),array(list(mapping)),"get_diag(vector)")

  def test_sparsityindex(self):
    self.message("sparsity indexing")
    nza = set([  (0,0),
             (0,1),
             (2,0),
             (2,3),
             (3,3),
             (2,4),
             (3,1),
             (4,1)])

    a = Sparsity(5,5)
    for i in nza:
      a.add_nz(i[0],i[1])

    b = MX.sym("b",a)

    self.assertRaises(Exception,lambda: b[Sparsity.diag(3)])

    d = Sparsity.diag(5)
    c = b[d]

    self.assertTrue(c.sparsity()==d)

    f = Function('f', [b],[c])
    fin = DM(b.sparsity(),list(range(1,len(nza)+1)))
    f_out = f(fin)

    self.checkarray(DM(f_out.nonzeros()),DM([1,0,0,7,0]),"sparsity index")

  def test_get_ccs(self):
    self.message("CCS format")
    nza = set([  (0,0),
             (0,1),
             (2,0),
             (2,3),
             (3,3),
             (2,4),
             (3,1)])

    a = Sparsity(4,5)
    for i in nza:
      a.add_nz(i[0],i[1])

    A1, B1= a.get_ccs()

    A2, B2 = (a.T).get_crs()

    print(A1, B1)
    print(A2, B2)

  def test_dm_diagcat_dense(self):
    self.message("Dulmage-Mendelsohn")
    random.seed(0)
    numpy.random.seed(0)
    for k in range(20):
      Ai = [self.randDM(d,d,1) for i,d in enumerate ([random.randint(1,10) for j in range(10)])]
      A = diagcat(*Ai)

      #A.sparsity().spy()
      perm =  numpy.random.permutation(list(range(A.size1())))

      AP = A[perm,perm]
      #AP.sparsity().spy()

      ret, rowperm, colperm, rowblock, colblock, coarse_rowblock, coarse_colblock = AP.sparsity().btf()

      Ar = AP[rowperm,colperm]

      ST = Ar.sparsity()

      blocks = []
      acc = -1
      mc = 0
      for i in range(0,Ar.size1()):
        mc = max(ST.row()[ST.colind()[i+1]-1],mc)
        if mc==i:
          blocks.append(i-acc)
          acc = i

      truth = [i.size1() for i in Ai]
      tryme = blocks

      truth.sort()
      tryme.sort()

      self.checkarray(truth,tryme)

  def test_scc_diagcat_sparse(self):
    self.message("scc")
    random.seed(0)
    numpy.random.seed(0)
    for k in range(20):
      Ai = [self.randDM(d,d,0.6,symm=True) for i,d in enumerate ([random.randint(1,10) for j in range(10)])]
      A = diagcat(*Ai)

      #A.sparsity().spy()
      perm =  numpy.random.permutation(list(range(A.size1())))

      AP = A[perm,perm]
      #AP.sparsity().spy()

      n,p,r = AP.sparsity().scc()

      Ar = AP[p,p]

      #print "permute"
      #Ar.sparsity().spy()

      ST = Ar.sparsity()

      blocks = []
      acc = -1
      mc = 0
      for i in range(0,Ar.size1()):
        mc = max(ST.row()[ST.colind()[i+1]-1],mc)
        if mc==i:
          blocks.append(i-acc)
          acc = i

      truth = [i.size1() for i in Ai]
      tryme = blocks

      self.assertTrue(n>=len(truth))
      self.assertTrue(n>=len(tryme))

  def test_dm(self):

    A = DM(6,4)
    A[0,0] = 1
    A[1,2] = 1
    A[2,2] = 1
    A[5,3] = 1

    ret, rowperm, colperm, rowblock, colblock, coarse_rowblock, coarse_colblock = A.sparsity().btf()

    # Checked with CSparse
    self.checkarray(DM([ret]),DM([4]))
    self.checkarray(rowperm,DM([2, 3, 4, 1, 0, 5]).T)
    self.checkarray(colperm,DM([ 2,0,3,1]).T)
    self.checkarray(rowblock,DM([ 0, 4,5,6,6]).T)
    self.checkarray(colblock,DM([ 0, 1,2,3,4]).T)
    self.checkarray(coarse_rowblock,DM([ 0, 3,4,6,6]).T)
    self.checkarray(coarse_colblock,DM([ 0, 1,3,3,4]).T)


    A = DM(6,4)
    A[0,0] = 1
    A[1,2] = 1
    A[2,2] = 1
    A[5,3] = 1
    A[4,1] = 1
    A[3,0] = 1

    A.sparsity().spy()

    ret, rowperm, colperm, rowblock, colblock, coarse_rowblock, coarse_colblock = A.sparsity().btf()

    # Checked with CSparse
    self.checkarray(DM([ret]),DM([3]))
    self.checkarray(rowperm,DM([2,3,0,1,4,5]).T)
    self.checkarray(colperm,DM([ 0, 2, 1, 3]).T)
    self.checkarray(rowblock,DM([ 0, 4,5,6]).T)
    self.checkarray(colblock,DM([ 0, 2,3,4]).T)
    self.checkarray(coarse_rowblock,DM([ 0, 2, 4,6,6]).T)
    self.checkarray(coarse_colblock,DM([ 0, 2,4,4,4]).T)

    A = DM(6,4)
    A[0,0] = 1
    A[1,2] = 1
    A[2,2] = 1
    A[5,3] = 1
    A[4,1] = 1
    A[3,0] = 1
    A = A + DM.eye(6)[:,:4]

    A.sparsity().spy()

    ret, rowperm, colperm, rowblock, colblock, coarse_rowblock, coarse_colblock = A.sparsity().btf()

    # Checked with CSparse
    self.checkarray(DM([ret]),DM([1]))
    self.checkarray(rowperm,DM([4, 5, 0, 1, 2, 3]).T)
    self.checkarray(colperm,DM([ 0, 1, 2, 3]).T)
    self.checkarray(rowblock,DM([ 0, 6]).T)
    self.checkarray(colblock,DM([ 0, 4]).T)
    self.checkarray(coarse_rowblock,DM([ 0, 2, 6,6,6]).T)
    self.checkarray(coarse_colblock,DM([ 0, 4,4,4,4]).T)


  def test_jacsparsityHierarchical(self):

    X = SX.sym("X",100)
    P = SX.sym("P",1000)

    optvar = vertcat(*[X,P])

    p = SX.sym("p")

    g = Function('g', [optvar,p],[X*p], {'verbose':True})

    J = jacobian_old(g, 0, 0)

    self.assertTrue(DM(J.sparsity_out(0))[:,:X.nnz()].sparsity()==Sparsity.diag(100))

    X = SX.sym("X",100)
    P = SX.sym("P",1000)

    p = SX.sym("p")

    g = Function('g', [X,P,p],[vertcat(*[X*p,P])], {'verbose':True})

    J = jacobian_old(g, 0, 0)

    self.assertTrue(DM(J.sparsity_out(0))[:X.nnz(),:].sparsity()==Sparsity.diag(100))

  @memory_heavy()
  def test_jacsparsityHierarchicalSymm(self):
    GlobalOptions.setHierarchicalSparsity(False)
    sp = Sparsity.banded(4129,1)

    x = MX.sym("x",sp.size1())

    H = bilin(MX.ones(sp),x,x)
    sp2 = hessian(H,x)[0].sparsity()
    self.assertTrue(sp==sp2)


  def test_rowcol(self):
    n = 3

    s = Sparsity.rowcol([n-1,0],[0,n-1],n,n)
    self.checkarray(s.colind(),[0,2,2,4])
    self.checkarray(s.row(),[0,2,0,2])

  def test_inverse(self):
    numpy.random.seed(0)
    d = self.randDM(20,20,0.6,symm=True)
    sp = d.sparsity()

    for sp in [sp,Sparsity.dense(4,4),Sparsity(4,4),Sparsity.lower(4),Sparsity.lower(4).T]:

      d = DM.ones(sp)

      dt = sparsify(1-d)
      dt = DM.ones(dt.sparsity())

      trial = DM.ones(sp.pattern_inverse())

      d.print_dense()
      dt.print_dense()
      trial.print_dense()

      self.checkarray(trial,dt)

  def test_kron(self):
    a = sparsify(DM([[1,0,6],[2,7,0]]))
    b = sparsify(DM([[1,0,0],[2,3,7],[0,0,9],[1,12,13]]))

    c_ = c.kron(a.sparsity(),b.sparsity())

    self.assertEqual(c_.size1(),a.size1()*b.size1())
    self.assertEqual(c_.size2(),a.size2()*b.size2())
    self.assertEqual(c_.nnz(),a.nnz()*b.nnz())

    self.checkarray(DM(c_,1),DM(c.kron(a,b).sparsity(),1))

  def test_nz_method(self):
    n = 20
    m = 25
    import random
    random.seed(0)
    numpy.random.seed(0)
    d = self.randDM(n,m,0.5)
    D = densify(vec(d))
    dn = DM(d.nonzeros())
    sp = d.sparsity()
    z = np.unique([random.randint(0,n*m-1) for i in range(200)])
    zres = sp.get_nz(z)
    A = dn[[e for e in zres if e>=0]]
    B = D[[e for e,k in zip(z,zres) if k>=0]]
    self.checkarray(A,B)
    self.assertFalse(np.any(D[[e for e,k in zip(z,zres) if k==-1]]))

  def test_serialize(self):
    for a in [Sparsity(), Sparsity.dense(4,5), Sparsity.lower(5)]:
      b = Sparsity.deserialize(a.serialize())
      if a.is_null():
        self.assertTrue(b.is_null())
      else:
        self.checkarray(DM(a,1),DM(b,1))

  def test_is_subset(self):

      pairs = [ (Sparsity.lower(3), Sparsity.dense(3,3)),
                (Sparsity.diag(3), Sparsity.dense(3,3)),
                (Sparsity.diag(3), Sparsity.lower(3)),
                (Sparsity(3,3), Sparsity.lower(3)),
      ]

      for L,R in pairs:
        self.assertTrue(L.is_subset(R))
        self.assertFalse(R.is_subset(L))



if __name__ == '__main__':
    unittest.main()



# This is an automatically generated file converting from the apm format

import casadi as cs

def hs025():
    # The optimal objective is (if given in):
    x_opt = cs.DM([50, 25, 1.5])
    f_opt = 0
    x = cs.MX.sym('x', 3)
    obj = cs.MX.zeros(99, 1)
    x0 = cs.DM.zeros(3, 1)
    lbx = -cs.inf*cs.DM.ones(3, 1)
    ubx = cs.inf*cs.DM.ones(3, 1)
    s = cs.MX.zeros(99, 1)
    u = cs.MX.zeros(99, 1)
    g = []
    lbg = []
    ubg = []
    
    x0[0]= 100
    x0[1]= 12.5
    x0[2]= 3
    
    s = cs.DM(range(1,100))
    u[0:99] = 25 + (-50*cs.log(s[0:99]/100))**(2/3)
    
    lbx[0] = 1/10
    ubx[0] =  100
    lbx[1] = 0
    ubx[1] =  25.6
    lbx[2] = 0
    ubx[2] =  5
    obj[0:99] = (-s[0:99]/100 + cs.exp(-(u[0:99] - x[1])**x[2]/x[0]))**2
    
    
    f = cs.sum1(obj)
    
    return (x_opt, f_opt, x, f, g, lbg, ubg, lbx, ubx, x0)



#
#     MIT No Attribution
#
#     Copyright (C) 2010-2023 Joel Andersson, Joris Gillis, Moritz Diehl, KU Leuven.
#
#     Permission is hereby granted, free of charge, to any person obtaining a copy of this
#     software and associated documentation files (the "Software"), to deal in the Software
#     without restriction, including without limitation the rights to use, copy, modify,
#     merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
#     permit persons to whom the Software is furnished to do so.
#
#     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
#     INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
#     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
#     HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
#     OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
#     SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
#
# -*- coding: utf-8 -*-
"""
@author: Mario Zanon and Sebastien Gross, KU Leuven 2012
"""

from casadi import *
import numpy as np
import matplotlib.pyplot as plt

# -----------------------------------------------------------------------------
# Collocation setup
# -----------------------------------------------------------------------------
nicp = 1        # Number of (intermediate) collocation points per control interval

xref = 0.1 # chariot reference

l = 1. #- -> crane, + -> pendulum
m = 1.
M = 1.
g = 9.81
tf = 5.0
nk = 50
ndstate = 6
nastate = 1
ninput = 1

# Degree of interpolating polynomial
deg = 4
# Radau collocation points
cp = "radau"
# Size of the finite elements
h = tf/nk/nicp

# Coefficients of the collocation equation
C = np.zeros((deg+1,deg+1))
# Coefficients of the continuity equation
D = np.zeros(deg+1)

# Collocation point
tau = SX.sym("tau")

# All collocation time points
tau_root = [0] + collocation_points(deg, cp)

T = np.zeros((nk,deg+1))
for i in range(nk):
    for j in range(deg+1):
        T[i][j] = h*(i + tau_root[j])

# For all collocation points: eq 10.4 or 10.17 in Biegler's book
# Construct Lagrange polynomials to get the polynomial basis at the collocation point
for j in range(deg+1):
    L = 1
    for j2 in range(deg+1):
        if j2 != j:
            L *= (tau-tau_root[j2])/(tau_root[j]-tau_root[j2])

    # Evaluate the polynomial at the final time to get the coefficients of the continuity equation
    lfcn = Function('lfcn', [tau],[L])
    D[j] = lfcn(1.0)

    # Evaluate the time derivative of the polynomial at all collocation points to get the coefficients of the continuity equation
    tfcn = Function('tfcn', [tau],[tangent(L,tau)])
    for j2 in range(deg+1):
        C[j][j2] = tfcn(tau_root[j2])

# -----------------------------------------------------------------------------
# Model setup
# -----------------------------------------------------------------------------
# Declare variables (use scalar graph)
t  = SX.sym("t")          # time
u  = SX.sym("u")          # control
xd  = SX.sym("xd",ndstate)      # differential state
xa  = SX.sym("xa",nastate)    # algebraic state
xddot  = SX.sym("xdot",ndstate) # differential state time derivative
p = SX.sym("p",0,1)      # parameters

x = SX.sym("x")
y = SX.sym("y")
w = SX.sym("w")

dx = SX.sym("dx")
dy = SX.sym("dy")
dw = SX.sym("dw")


res = vertcat(xddot[0] - dx,\
       xddot[1] - dy,\
       xddot[2] - dw,\
       m*xddot[3] + (x-w)*xa, \
       m*xddot[4] +     y*xa - g*m,\
       M*xddot[5] + (w-x)*xa +   u,\
       (x-w)*(xddot[3] - xddot[5]) + y*xddot[4] + dy*dy + (dx-dw)*(dx-dw))


xd[0] = x
xd[1] = y
xd[2] = w
xd[3] = dx
xd[4] = dy
xd[5] = dw


# System dynamics (implicit formulation)
ffcn = Function('ffcn', [t,xddot,xd,xa,u,p],[res])

# Objective function
MayerTerm = Function('mayer', [t,xd,xa,u,p],[(x-xref)*(x-xref) + (w-xref)*(w-xref) + dx*dx + dy*dy])
LagrangeTerm = Function('lagrange', [t,xd,xa,u,p],[(x-xref)*(x-xref) + (w-xref)*(w-xref)])

# Control bounds
u_min = np.array([-2])
u_max = np.array([ 2])
u_init = np.array((nk*nicp*(deg+1))*[[0.0]]) # needs to be specified for every time interval (even though it stays constant)

# Differential state bounds
#Path bounds
xD_min =  np.array([-inf, -inf, -inf, -inf, -inf, -inf])
xD_max =  np.array([ inf,  inf,  inf,  inf,  inf,  inf])
#Initial bounds
xDi_min = np.array([ 0.0,  l,  0.0,  0.0,  0.0,  0.0])
xDi_max = np.array([ 0.0,  l,  0.0,  0.0,  0.0,  0.0])
#Final bounds
xDf_min = np.array([-inf, -inf, -inf, -inf, -inf, -inf])
xDf_max = np.array([ inf,  inf,  inf,  inf,  inf,  inf])

#Initial guess for differential states
xD_init = np.array((nk*nicp*(deg+1))*[[ 0.0,  l,  0.0,  0.0,  0.0,  0.0]]) # needs to be specified for every time interval

# Algebraic state bounds and initial guess
xA_min =  np.array([-inf])
xA_max =  np.array([ inf])
xAi_min = np.array([-inf])
xAi_max = np.array([ inf])
xAf_min = np.array([-inf])
xAf_max = np.array([ inf])
xA_init = np.array((nk*nicp*(deg+1))*[[sign(l)*9.81]])

# Parameter bounds and initial guess
p_min = np.array([])
p_max = np.array([])
p_init = np.array([])




# -----------------------------------------------------------------------------
# Constraints setup
# -----------------------------------------------------------------------------
# Initial constraint
ic_min = np.array([])
ic_max = np.array([])
ic = SX()
#ic.append();       ic_min = append(ic_min, 0.);         ic_max = append(ic_max, 0.)
icfcn = Function('icfcn', [t,xd,xa,u,p],[ic])
# Path constraint
pc_min = np.array([])
pc_max = np.array([])
pc = SX()
#pc.append();       pc_min = append(pc_min, 0.);         pc_max = append(pc_max, 0.)
pcfcn = Function('pcfcn', [t,xd,xa,u,p],[pc])
# Final constraint
fc_min = np.array([])
fc_max = np.array([])
fc = SX()
#fc.append();       fc_min = append(fc_min, 0.);         fc_max = append(fc_max, 0.)
fcfcn = Function('fcfcn', [t,xd,xa,u,p],[fc])

# -----------------------------------------------------------------------------
# NLP setup
# -----------------------------------------------------------------------------
# Dimensions of the problem
nx = xd.nnz() + xa.nnz()  # total number of states        #MODIF
ndiff = xd.nnz()           # number of differential states #MODIF
nalg = xa.nnz()            # number of algebraic states
nu = u.nnz()               # number of controls
NP  = p.nnz()              # number of parameters

# Total number of variables
NXD = nicp*nk*(deg+1)*ndiff # Collocated differential states
NXA = nicp*nk*deg*nalg      # Collocated algebraic states
NU = nk*nu                  # Parametrized controls
NXF = ndiff                 # Final state (only the differential states)
NV = NXD+NXA+NU+NXF+NP

# NLP variable vector
V = MX.sym("V",NV)

# All variables with bounds and initial guess
vars_lb = np.zeros(NV)
vars_ub = np.zeros(NV)
vars_init = np.zeros(NV)
offset = 0

# Get the parameters
P = V[offset:offset+NP]
vars_init[offset:offset+NP] = p_init
vars_lb[offset:offset+NP] = p_min
vars_ub[offset:offset+NP] = p_max
offset += NP

# Get collocated states and parametrized control
XD = np.resize(np.array([],dtype=MX),(nk+1,nicp,deg+1)) # NB: same name as above
XA = np.resize(np.array([],dtype=MX),(nk,nicp,deg)) # NB: same name as above
U = np.resize(np.array([],dtype=MX),nk)
for k in range(nk):
    # Collocated states
    for i in range(nicp):
        #
        for j in range(deg+1):

            # Get the expression for the state vector
            XD[k][i][j] = V[offset:offset+ndiff]
            if j !=0:
                XA[k][i][j-1] = V[offset+ndiff:offset+ndiff+nalg]
            # Add the initial condition
            index = (deg+1)*(nicp*k+i) + j
            if k==0 and j==0 and i==0:
                vars_init[offset:offset+ndiff] = xD_init[index,:]

                vars_lb[offset:offset+ndiff] = xDi_min
                vars_ub[offset:offset+ndiff] = xDi_max
                offset += ndiff
            else:
                if j!=0:
                    vars_init[offset:offset+nx] = np.append(xD_init[index,:],xA_init[index,:])

                    vars_lb[offset:offset+nx] = np.append(xD_min,xA_min)
                    vars_ub[offset:offset+nx] = np.append(xD_max,xA_max)
                    offset += nx
                else:
                    vars_init[offset:offset+ndiff] = xD_init[index,:]

                    vars_lb[offset:offset+ndiff] = xD_min
                    vars_ub[offset:offset+ndiff] = xD_max
                    offset += ndiff

    # Parametrized controls
    U[k] = V[offset:offset+nu]
    vars_lb[offset:offset+nu] = u_min
    vars_ub[offset:offset+nu] = u_max
    vars_init[offset:offset+nu] = u_init[index,:]
    offset += nu

# State at end time
XD[nk][0][0] = V[offset:offset+ndiff]
vars_lb[offset:offset+ndiff] = xDf_min
vars_ub[offset:offset+ndiff] = xDf_max
vars_init[offset:offset+ndiff] = xD_init[-1,:]
offset += ndiff
assert(offset==NV)

# Constraint function for the NLP
g = []
lbg = []
ubg = []

# Initial constraints
ick = icfcn(0., XD[0][0][0], XA[0][0][0], U[0], P)
g += [ick]
lbg.append(ic_min)
ubg.append(ic_max)

# For all finite elements
for k in range(nk):
    for i in range(nicp):
        # For all collocation points
        for j in range(1,deg+1):
            # Get an expression for the state derivative at the collocation point
            xp_jk = 0
            for j2 in range (deg+1):
                xp_jk += C[j2][j]*XD[k][i][j2]       # get the time derivative of the differential states (eq 10.19b)

            # Add collocation equations to the NLP
            fk = ffcn(0., xp_jk/h, XD[k][i][j], XA[k][i][j-1], U[k], P)
            g += [fk[:ndiff]]                     # impose system dynamics (for the differential states (eq 10.19b))
            lbg.append(np.zeros(ndiff)) # equality constraints
            ubg.append(np.zeros(ndiff)) # equality constraints
            g += [fk[ndiff:]]                               # impose system dynamics (for the algebraic states (eq 10.19b))
            lbg.append(np.zeros(nalg)) # equality constraints
            ubg.append(np.zeros(nalg)) # equality constraints

            #  Evaluate the path constraint function
            pck = pcfcn(0., XD[k][i][j], XA[k][i][j-1], U[k], P)

            g += [pck]
            lbg.append(pc_min)
            ubg.append(pc_max)

        # Get an expression for the state at the end of the finite element
        xf_k = 0
        for j in range(deg+1):
            xf_k += D[j]*XD[k][i][j]

        # Add continuity equation to NLP
        if i==nicp-1:
#            print "a ", k, i
            g += [XD[k+1][0][0] - xf_k]
        else:
#            print "b ", k, i
            g += [XD[k][i+1][0] - xf_k]

        lbg.append(np.zeros(ndiff))
        ubg.append(np.zeros(ndiff))

# Periodicity constraints
#   none

# Final constraints (Const, dConst, ConstQ)
fck = fcfcn(0., XD[k][i][j], XA[k][i][j-1], U[k], P)
g += [fck]
lbg.append(fc_min)
ubg.append(fc_max)

# Objective function of the NLP
#Implement Mayer term
Obj = 0
obj = MayerTerm(0., XD[k][i][j], XA[k][i][j-1], U[k], P)
Obj += obj

# Implement Lagrange term
lDotAtTauRoot = C.T
lAtOne = D

ldInv = np.linalg.inv(lDotAtTauRoot[1:,1:])
ld0 = lDotAtTauRoot[1:,0]
lagrangeTerm = 0
for k in range(nk):
    for i in range(nicp):
        dQs = h*veccat(*[LagrangeTerm(0., XD[k][i][j], XA[k][i][j-1], U[k], P) \
                        for j in range(1,deg+1)])
        Qs = mtimes( ldInv, dQs)
        m = mtimes( Qs.T, lAtOne[1:])
        lagrangeTerm += m

Obj += lagrangeTerm

# NLP
nlp = {'x':V, 'f':Obj, 'g':vertcat(*g)}

## ----
## SOLVE THE NLP
## ----

# NLP solver options
opts = {}
opts["expand"] = True
opts["ipopt.max_iter"] = 1000
opts["ipopt.tol"] = 1e-4
opts["ipopt.linear_solver"] = 'ma27'

# Allocate an NLP solver
solver = nlpsol("solver", "ipopt", nlp, opts)
arg = {}

# Initial condition
arg["x0"] = vars_init

# Bounds on x
arg["lbx"] = vars_lb
arg["ubx"] = vars_ub

# Bounds on g
arg["lbg"] = np.concatenate(lbg)
arg["ubg"] = np.concatenate(ubg)

# Solve the problem
res = solver(**arg)

# Print the optimal cost
print("optimal cost: ", float(res["f"]))

# Retrieve the solution
v_opt = np.array(res["x"])


## ----
## RETRIEVE THE SOLUTION
## ----
xD_opt = np.resize(np.array([],dtype=MX),(ndiff,(deg+1)*nicp*(nk)+1))
xA_opt = np.resize(np.array([],dtype=MX),(nalg,(deg)*nicp*(nk)))
u_opt = np.resize(np.array([],dtype=MX),(nu,(deg+1)*nicp*(nk)+1))
offset = 0
offset2 = 0
offset3 = 0
offset4 = 0

for k in range(nk):
    for i in range(nicp):
        for j in range(deg+1):
            xD_opt[:,offset2] = v_opt[offset:offset+ndiff][:,0]
            offset2 += 1
            offset += ndiff
            if j!=0:
                xA_opt[:,offset4] = v_opt[offset:offset+nalg][:,0]
                offset4 += 1
                offset += nalg
    utemp = v_opt[offset:offset+nu][:,0]
    for i in range(nicp):
        for j in range(deg+1):
            u_opt[:,offset3] = utemp
            offset3 += 1
    #    u_opt += v_opt[offset:offset+nu]
    offset += nu

xD_opt[:,-1] = v_opt[offset:offset+ndiff][:,0]



# The algebraic states are not defined at the first collocation point of the finite elements:
# with the polynomials we compute them at that point
Da = np.zeros(deg)
for j in range(1,deg+1):
    # Lagrange polynomials for the algebraic states: exclude the first point
    La = 1
    for j2 in range(1,deg+1):
        if j2 != j:
            La *= (tau-tau_root[j2])/(tau_root[j]-tau_root[j2])
    lafcn = Function('lafcn', [tau], [La])
    Da[j-1] = lafcn(tau_root[0])

xA_plt = np.resize(np.array([],dtype=MX),(nalg,(deg+1)*nicp*(nk)+1))
offset4=0
offset5=0
for k in range(nk):
    for i in range(nicp):
        for j in range(deg+1):
            if j!=0:
                xA_plt[:,offset5] = xA_opt[:,offset4]
                offset4 += 1
                offset5 += 1
            else:
                xa0 = 0
                for j in range(deg):
                    xa0 += Da[j]*xA_opt[:,offset4+j]
                xA_plt[:,offset5] = xa0
                #xA_plt[:,offset5] = xA_opt[:,offset4]
                offset5 += 1

xA_plt[:,-1] = xA_plt[:,-2]




tg = np.array(tau_root)*h
for k in range(nk*nicp):
    if k == 0:
        tgrid = tg
    else:
        tgrid = np.append(tgrid,tgrid[-1]+tg)
tgrid = np.append(tgrid,tgrid[-1])
# Plot the results
plt.figure(1)
plt.clf()
plt.subplot(2,2,1)
plt.plot(tgrid,xD_opt[0,:],'--')
plt.title("x")
plt.grid
plt.subplot(2,2,2)
plt.plot(tgrid,xD_opt[1,:],'-')
plt.title("y")
plt.grid
plt.subplot(2,2,3)
plt.plot(tgrid,xD_opt[2,:],'-.')
plt.title("w")
plt.grid

plt.figure(2)
plt.clf()
plt.plot(tgrid,u_opt[0,:],'-.')
plt.title("Crane, inputs")
plt.xlabel('time')


plt.figure(3)
plt.clf()
plt.plot(tgrid,xA_plt[0,:],'-.')
plt.title("Crane, lambda")
plt.xlabel('time')
plt.grid()
plt.show()
